import os
from typing import Union


class Unset:
    def __bool__(self) -> bool:
        return False


UNSET: Unset = Unset()


def is_unset(obj) -> bool:
    return isinstance(obj, Unset)


def is_set(obj) -> bool:
    return not is_unset(obj)


PathLike = Union[str, os.PathLike]



from enum import Enum


class NlpLanguage(str, Enum):
    AUTO_DETECT = "AUTO_DETECT"
    ARABIC = "ARABIC"
    DANISH = "DANISH"
    DUTCH = "DUTCH"
    ENGLISH = "ENGLISH"
    FINNISH = "FINNISH"
    FRENCH = "FRENCH"
    GERMAN = "GERMAN"
    HUNGARIAN = "HUNGARIAN"
    ITALIAN = "ITALIAN"
    NORWEGIAN = "NORWEGIAN"
    PORTUGUESE = "PORTUGUESE"
    ROMANIAN = "ROMANIAN"
    RUSSIAN = "RUSSIAN"
    SPANISH = "SPANISH"
    SWEDISH = "SWEDISH"
    NONE = "NONE"

    def __str__(self) -> str:
        return str(self.value)


from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.combine_step_step_type import CombineStepStepType

T = TypeVar("T", bound="CombineStep")

@attr.s(auto_attribs=True)
class CombineStep:
    """  """
    column: str
    data_set_column: str
    data_set_id: int
    step_type: CombineStepStepType
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        column =  self.column
        data_set_column =  self.data_set_column
        data_set_id =  self.data_set_id
        step_type = self.step_type.value


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "column": column,
            "dataSetColumn": data_set_column,
            "dataSetId": data_set_id,
            "stepType": step_type,
        })

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        column = d.pop("column")

        data_set_column = d.pop("dataSetColumn")

        data_set_id = d.pop("dataSetId")

        step_type = CombineStepStepType(d.pop("stepType"))




        combine_step = cls(
            column=column,
            data_set_column=data_set_column,
            data_set_id=data_set_id,
            step_type=step_type,
        )

        combine_step.additional_properties = d
        return combine_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import List, Optional

import attr

from ....swagger.api.models import create_classification_model
from ....swagger.models import (
    ClassificationScorers,
    CreateClassificationRequest,
    DataPartitionMethod,
    ModelValidationMethod,
    NlpLanguage,
)
from ....swagger.types import UNSET
from ....utils import check_exists
from ...connection import ConnectionLike, client_from_connectionlike
from ...data_set import DataLike, get_dataset_from_datalike
from ..shared import BaseModel, LocalModel, RemoteModel


@attr.s(auto_attribs=True)
class InitialConfig:
    scorer: ClassificationScorers
    model_validation_method: ModelValidationMethod
    name: str
    description: Optional[str]
    number_of_evaluations: int
    nlp_language: NlpLanguage
    draft_mode: bool


class LocalClassifier(LocalModel):
    """
    A classifier that has not yet been saved to Analyze (and thus hasn't been fitted)
    """

    def __init__(
        self,
        connection: ConnectionLike,
        initial_config: InitialConfig,
    ):
        super().__init__()
        self._connection = connection
        self._client = client_from_connectionlike(connection)
        self._initial_config = initial_config

    def fit(
        self,
        data: DataLike,
        target: str,
        excluded_columns: Optional[List[str]] = None,
        *,
        order_by: Optional[str] = None,
        partition_by: Optional[str] = None,
        no_mixing: Optional[List[str]] = None,
    ) -> BaseModel:

        # data_partition_method - determine based on whether there are orderby/partitionby/nomixing names
        if not order_by is None:
            data_partition_method = DataPartitionMethod.ORDERED
        elif not partition_by is None:
            data_partition_method = DataPartitionMethod.MANUAL
        elif not no_mixing is None:
            data_partition_method = DataPartitionMethod.NO_MIXING
        else:
            data_partition_method = DataPartitionMethod.RANDOM

        data_set = get_dataset_from_datalike(data, connection=self._connection)

        payload = CreateClassificationRequest(
            name=self._initial_config.name,
            data_set_id=data_set.data_set_id,
            target_column=target,
            score_to_optimize=self._initial_config.scorer,
            model_validation_method=self._initial_config.model_validation_method,
            data_partition_method=data_partition_method,
            number_of_evaluations=self._initial_config.number_of_evaluations,
            description=self._initial_config.description or UNSET,
            excluded_columns=excluded_columns or UNSET,
            nlp_language=self._initial_config.nlp_language,
            draft_mode=self._initial_config.draft_mode,
            no_mixing_columns=no_mixing or UNSET,
            partition_column=partition_by or UNSET,
            order_by_column=order_by or UNSET,
        )

        model_id = check_exists(
            create_classification_model.sync(client=self._client, json_body=payload)
        ).id
        return RemoteModel(self._connection, model_id)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.deployed_api_client_response import DeployedApiClientResponse
from ..types import UNSET, Unset

T = TypeVar("T", bound="DeployedApiClientListResponse")

@attr.s(auto_attribs=True)
class DeployedApiClientListResponse:
    """  """
    api_clients: Union[Unset, List[DeployedApiClientResponse]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        api_clients: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.api_clients, Unset):
            api_clients = []
            for api_clients_item_data in self.api_clients:
                api_clients_item = api_clients_item_data.to_dict()

                api_clients.append(api_clients_item)





        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if api_clients is not UNSET:
            field_dict["apiClients"] = api_clients

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        api_clients = []
        _api_clients = d.pop("apiClients", UNSET)
        for api_clients_item_data in (_api_clients or []):
            api_clients_item = DeployedApiClientResponse.from_dict(api_clients_item_data)



            api_clients.append(api_clients_item)


        deployed_api_client_list_response = cls(
            api_clients=api_clients,
        )

        deployed_api_client_list_response.additional_properties = d
        return deployed_api_client_list_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.transform_text_step_if_no_match import TransformTextStepIfNoMatch
from ..models.transform_text_step_replace_existing_column import (
    TransformTextStepReplaceExistingColumn,
)
from ..models.transform_text_step_step_type import TransformTextStepStepType
from ..types import UNSET, Unset

T = TypeVar("T", bound="TransformTextStep")

@attr.s(auto_attribs=True)
class TransformTextStep:
    """  """
    column: str
    input_regex: str
    if_no_match: TransformTextStepIfNoMatch
    step_type: TransformTextStepStepType
    ignore_case: Union[Unset, bool] = UNSET
    output_regex: Union[Unset, str] = UNSET
    new_column_name: Union[Unset, str] = UNSET
    replace_existing_column: Union[Unset, TransformTextStepReplaceExistingColumn] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        column =  self.column
        input_regex =  self.input_regex
        if_no_match = self.if_no_match.value

        step_type = self.step_type.value

        ignore_case =  self.ignore_case
        output_regex =  self.output_regex
        new_column_name =  self.new_column_name
        replace_existing_column: Union[Unset, TransformTextStepReplaceExistingColumn] = UNSET
        if not isinstance(self.replace_existing_column, Unset):
            replace_existing_column = self.replace_existing_column


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "column": column,
            "inputRegex": input_regex,
            "ifNoMatch": if_no_match,
            "stepType": step_type,
        })
        if ignore_case is not UNSET:
            field_dict["ignoreCase"] = ignore_case
        if output_regex is not UNSET:
            field_dict["outputRegex"] = output_regex
        if new_column_name is not UNSET:
            field_dict["newColumnName"] = new_column_name
        if replace_existing_column is not UNSET:
            field_dict["replaceExistingColumn"] = replace_existing_column

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        column = d.pop("column")

        input_regex = d.pop("inputRegex")

        if_no_match = TransformTextStepIfNoMatch(d.pop("ifNoMatch"))




        step_type = TransformTextStepStepType(d.pop("stepType"))




        ignore_case = d.pop("ignoreCase", UNSET)

        output_regex = d.pop("outputRegex", UNSET)

        new_column_name = d.pop("newColumnName", UNSET)

        replace_existing_column: Union[Unset, TransformTextStepReplaceExistingColumn] = UNSET
        _replace_existing_column = d.pop("replaceExistingColumn", UNSET)
        if not isinstance(_replace_existing_column,  Unset):
            replace_existing_column = TransformTextStepReplaceExistingColumn(_replace_existing_column)




        transform_text_step = cls(
            column=column,
            input_regex=input_regex,
            if_no_match=if_no_match,
            step_type=step_type,
            ignore_case=ignore_case,
            output_regex=output_regex,
            new_column_name=new_column_name,
            replace_existing_column=replace_existing_column,
        )

        transform_text_step.additional_properties = d
        return transform_text_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.simple_forecast_cadence import SimpleForecastCadence
from ..types import UNSET, Unset

T = TypeVar("T", bound="SimpleForecast")

@attr.s(auto_attribs=True)
class SimpleForecast:
    """  """
    horizon: Union[Unset, int] = UNSET
    cadence: Union[Unset, SimpleForecastCadence] = UNSET
    index: Union[Unset, datetime.datetime] = UNSET
    predicted_value: Union[Unset, float] = UNSET
    lower_bound: Union[Unset, float] = UNSET
    upper_bound: Union[Unset, float] = UNSET
    historic_rmse_score: Union[Unset, float] = UNSET
    historic_r2_score: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        horizon =  self.horizon
        cadence: Union[Unset, SimpleForecastCadence] = UNSET
        if not isinstance(self.cadence, Unset):
            cadence = self.cadence

        index: Union[Unset, str] = UNSET
        if not isinstance(self.index, Unset):
            index = self.index.isoformat()

        predicted_value =  self.predicted_value
        lower_bound =  self.lower_bound
        upper_bound =  self.upper_bound
        historic_rmse_score =  self.historic_rmse_score
        historic_r2_score =  self.historic_r2_score

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if horizon is not UNSET:
            field_dict["horizon"] = horizon
        if cadence is not UNSET:
            field_dict["cadence"] = cadence
        if index is not UNSET:
            field_dict["index"] = index
        if predicted_value is not UNSET:
            field_dict["predictedValue"] = predicted_value
        if lower_bound is not UNSET:
            field_dict["lowerBound"] = lower_bound
        if upper_bound is not UNSET:
            field_dict["upperBound"] = upper_bound
        if historic_rmse_score is not UNSET:
            field_dict["historicRmseScore"] = historic_rmse_score
        if historic_r2_score is not UNSET:
            field_dict["historicR2Score"] = historic_r2_score

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        horizon = d.pop("horizon", UNSET)

        cadence: Union[Unset, SimpleForecastCadence] = UNSET
        _cadence = d.pop("cadence", UNSET)
        if not isinstance(_cadence,  Unset):
            cadence = SimpleForecastCadence(_cadence)




        index: Union[Unset, datetime.datetime] = UNSET
        _index = d.pop("index", UNSET)
        if not isinstance(_index,  Unset):
            index = isoparse(_index)




        predicted_value = d.pop("predictedValue", UNSET)

        lower_bound = d.pop("lowerBound", UNSET)

        upper_bound = d.pop("upperBound", UNSET)

        historic_rmse_score = d.pop("historicRmseScore", UNSET)

        historic_r2_score = d.pop("historicR2Score", UNSET)

        simple_forecast = cls(
            horizon=horizon,
            cadence=cadence,
            index=index,
            predicted_value=predicted_value,
            lower_bound=lower_bound,
            upper_bound=upper_bound,
            historic_rmse_score=historic_rmse_score,
            historic_r2_score=historic_r2_score,
        )

        simple_forecast.additional_properties = d
        return simple_forecast

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# The sklearn-like API
# Types for the sklearn-like API
from ..utils.typing import PathLike
from . import model, prediction, test
from .connection import Connection, ConnectionLike
from .data_set import DataLike, DataSet, load_data_set_from_server
from .model import load_model, load_model_from_server
from .prediction import load_prediction_from_server
from .test import load_test_from_server



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.create_http_data_set_request_data_origin import (
    CreateHttpDataSetRequestDataOrigin,
)
from ..models.parse_options import ParseOptions
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateHttpDataSetRequest")

@attr.s(auto_attribs=True)
class CreateHttpDataSetRequest:
    """  """
    name: str
    description: str
    url: str
    data_origin: CreateHttpDataSetRequestDataOrigin
    parse_options: Union[ParseOptions, Unset] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        name =  self.name
        description =  self.description
        url =  self.url
        data_origin = self.data_origin.value

        parse_options: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parse_options, Unset):
            parse_options = self.parse_options.to_dict()


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "name": name,
            "description": description,
            "url": url,
            "dataOrigin": data_origin,
        })
        if parse_options is not UNSET:
            field_dict["parseOptions"] = parse_options

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        description = d.pop("description")

        url = d.pop("url")

        data_origin = CreateHttpDataSetRequestDataOrigin(d.pop("dataOrigin"))




        parse_options: Union[ParseOptions, Unset] = UNSET
        _parse_options = d.pop("parseOptions", UNSET)
        if not isinstance(_parse_options,  Unset):
            parse_options = ParseOptions.from_dict(_parse_options)




        create_http_data_set_request = cls(
            name=name,
            description=description,
            url=url,
            data_origin=data_origin,
            parse_options=parse_options,
        )

        create_http_data_set_request.additional_properties = d
        return create_http_data_set_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.prediction_response import PredictionResponse
from ..types import UNSET, Unset

T = TypeVar("T", bound="PredictionListResponse")

@attr.s(auto_attribs=True)
class PredictionListResponse:
    """  """
    predictions: Union[Unset, List[PredictionResponse]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        predictions: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.predictions, Unset):
            predictions = []
            for predictions_item_data in self.predictions:
                predictions_item = predictions_item_data.to_dict()

                predictions.append(predictions_item)





        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if predictions is not UNSET:
            field_dict["predictions"] = predictions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        predictions = []
        _predictions = d.pop("predictions", UNSET)
        for predictions_item_data in (_predictions or []):
            predictions_item = PredictionResponse.from_dict(predictions_item_data)



            predictions.append(predictions_item)


        prediction_list_response = cls(
            predictions=predictions,
        )

        prediction_list_response.additional_properties = d
        return prediction_list_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DataPartitionMethod(str, Enum):
    RANDOM = "RANDOM"
    ORDERED = "ORDERED"
    MANUAL = "MANUAL"
    NO_MIXING = "NO_MIXING"

    def __str__(self) -> str:
        return str(self.value)


import json
from typing import Union

from mindfoundry.client.analyze.client import create_project_api_swagger_client
from mindfoundry.client.analyze.swagger import Client
from mindfoundry.client.analyze.utils.typing import PathLike


class Connection:
    def __init__(self, base_url: str, client_id: str, client_secret: str):
        self.base_url = base_url.rstrip("/")
        self.client_id = client_id
        self.client_secret = client_secret

    def to_file(self, path: PathLike) -> None:
        dict_repr = {
            "connection": {
                "base_url": self.base_url,
                "client_id": self.client_id,
                "client_secret": self.client_secret,
            }
        }
        with open(path, "w") as file:
            json.dump(dict_repr, file)

    @classmethod
    def from_file(cls, path: PathLike) -> "Connection":
        with open(path, mode="r") as file:
            config = json.load(file)

            if "connection" not in config.keys():
                raise Exception("Not a connection")
            connection_config = config["connection"]

            if "base_url" not in connection_config.keys():
                raise Exception(
                    "Configuration did not represent an Analyze connection - No base_url saved"
                )
            base_url = connection_config["base_url"]

            if "client_id" not in connection_config.keys():
                raise Exception(
                    "Configuration did not represent an Analyze connection - No client_id saved"
                )
            client_id = connection_config["client_id"]

            if "client_secret" not in connection_config.keys():
                raise Exception(
                    "Configuration did not represent an Analyze connection - No client_secret saved"
                )
            client_secret = connection_config["client_secret"]

        return Connection(base_url, client_id, client_secret)


ConnectionLike = Union[PathLike, "Connection"]


def from_connectionlike(connection: ConnectionLike) -> Connection:
    if isinstance(connection, Connection):
        return connection
    # If we've been given a pathlike, load the connection information from the pathlike
    return Connection.from_file(connection)


def client_from_connectionlike(connection: ConnectionLike) -> Client:
    connection = from_connectionlike(connection)
    return create_project_api_swagger_client(
        connection.base_url,
        connection.client_id,
        connection.client_secret,
    )



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.apply_formula_step_step_type import ApplyFormulaStepStepType

T = TypeVar("T", bound="ApplyFormulaStep")

@attr.s(auto_attribs=True)
class ApplyFormulaStep:
    """  """
    formula: str
    result_column_name: str
    step_type: ApplyFormulaStepStepType
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        formula =  self.formula
        result_column_name =  self.result_column_name
        step_type = self.step_type.value


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "formula": formula,
            "resultColumnName": result_column_name,
            "stepType": step_type,
        })

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        formula = d.pop("formula")

        result_column_name = d.pop("resultColumnName")

        step_type = ApplyFormulaStepStepType(d.pop("stepType"))




        apply_formula_step = cls(
            formula=formula,
            result_column_name=result_column_name,
            step_type=step_type,
        )

        apply_formula_step.additional_properties = d
        return apply_formula_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    id: int,

) -> Dict[str, Any]:
    url = "{}/v1/dataPipelines/{id}".format(
        client.base_url,id=id)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    

    

    

    

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[
    None,
    None,
    None,
    None
]]:
    if response.status_code == 204:
        response_204 = None

        return response_204
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 404:
        response_404 = None

        return response_404
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[
    None,
    None,
    None,
    None
]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    id: int,

) -> Response[Union[
    None,
    None,
    None,
    None
]]:
    kwargs = _get_kwargs(
        client=client,
id=id,

    )

    response = httpx.delete(
        **kwargs,
    )

    return _build_response(response=response)

def sync(
    *,
    client: Client,
    id: int,

) -> Optional[Union[
    None,
    None,
    None,
    None
]]:
    """ Deletes the specified data prep pipeline """

    return sync_detailed(
        client=client,
id=id,

    ).parsed

async def asyncio_detailed(
    *,
    client: Client,
    id: int,

) -> Response[Union[
    None,
    None,
    None,
    None
]]:
    kwargs = _get_kwargs(
        client=client,
id=id,

    )

    async with httpx.AsyncClient() as _client:
        response = await _client.delete(
            **kwargs
        )

    return _build_response(response=response)

async def asyncio(
    *,
    client: Client,
    id: int,

) -> Optional[Union[
    None,
    None,
    None,
    None
]]:
    """ Deletes the specified data prep pipeline """

    return (await asyncio_detailed(
        client=client,
id=id,

    )).parsed



from typing import Any, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..models.future_outputofyourrequest import FutureOutputofyourrequest
from ..models.status import Status
from ..types import UNSET, Unset

T = TypeVar("T", bound="Future")

@attr.s(auto_attribs=True)
class Future:
    """  """
    future_id: Union[Unset, str] = UNSET
    status: Union[Unset, Status] = UNSET
    response: Union[Optional[FutureOutputofyourrequest], Unset] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        future_id =  self.future_id
        status: Union[Unset, Status] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status

        response: Union[None, Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.response, Unset):
            response = self.response.to_dict() if self.response else None


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if future_id is not UNSET:
            field_dict["futureId"] = future_id
        if status is not UNSET:
            field_dict["status"] = status
        if response is not UNSET:
            field_dict["response"] = response

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        future_id = d.pop("futureId", UNSET)

        status: Union[Unset, Status] = UNSET
        _status = d.pop("status", UNSET)
        if not isinstance(_status,  Unset):
            status = Status(_status)




        response = None
        _response = d.pop("response", UNSET)
        if _response is not None and not isinstance(_response,  Unset):
            response = FutureOutputofyourrequest.from_dict(_response)




        future = cls(
            future_id=future_id,
            status=status,
            response=response,
        )

        future.additional_properties = d
        return future

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Union

from ...swagger.api.models import get_model
from ...swagger.models import ProblemType
from ...utils import check_exists
from ...utils.typing import PathLike
from ..connection import ConnectionLike, client_from_connectionlike
from . import AnalyzeClassifier
from .regressor.analyze_regressor import AnalyzeRegressor
from .serialization import parse_model_from_path


def load_model(
    path: PathLike, *, connection: ConnectionLike
) -> Union[AnalyzeRegressor, AnalyzeClassifier]:
    serialized_model = parse_model_from_path(path)

    if serialized_model.problem_type == ProblemType.REGRESSION:
        return AnalyzeRegressor.load_model(path, connection=connection)

    if serialized_model.problem_type == ProblemType.CLASSIFICATION:
        return AnalyzeClassifier.load_model(path, connection=connection)

    raise Exception("Unsupported model type")


def _work_out_model_type(connection: ConnectionLike, model_id: int):
    client = client_from_connectionlike(connection)
    model_info = check_exists(get_model.sync(client=client, id=model_id))
    return model_info.problem_type


def load_model_from_server(
    model_id: int, *, connection: ConnectionLike
) -> Union[AnalyzeRegressor, AnalyzeClassifier]:
    problem_type = _work_out_model_type(connection, model_id)

    if problem_type == ProblemType.REGRESSION:
        return AnalyzeRegressor.load_model_from_server(model_id, connection=connection)
    if problem_type == ProblemType.CLASSIFICATION:
        return AnalyzeClassifier.load_model_from_server(model_id, connection=connection)
    raise Exception("Unsupported model type")



import io
import json
import tempfile
from os import PathLike
from typing import List, Optional, Union

import pandas as pd

from .swagger import Client
from .swagger import models as dto
from .swagger.api.ap_is import (
    create_model_api,
    create_model_api_client,
    create_pipeline_api,
    create_pipeline_api_client,
    delete_model_api,
    delete_model_api_client,
    delete_pipeline_api,
    delete_pipeline_api_client,
    edit_model_api,
    edit_pipeline_api,
    get_model_api,
    get_model_api_client,
    get_model_api_clients,
    get_model_apis,
    get_pipeline_api,
    get_pipeline_api_client,
    get_pipeline_api_clients,
    get_pipeline_apis,
)
from .swagger.api.data import (
    apply_data_prep_steps,
    create_cloud_data_set,
    create_db_data_set,
    create_file_data_set,
    create_http_data_set,
    delete_data_prep_steps,
    delete_data_set,
    edit_data_set,
    get_data_set,
    get_data_sets,
    get_data_sets_csv_data,
    get_data_sets_parquet_data,
)
from .swagger.api.models import (
    create_auto_clustering_model,
    create_classification_model,
    create_clustering_model,
    create_forecasting_model,
    create_multi_forecasting_model,
    create_regression_model,
    delete_model,
    edit_model,
    get_model,
    get_models,
    stop_model,
)
from .swagger.api.pipelines import (
    create_data_pipeline,
    delete_data_pipeline,
    edit_data_pipeline,
    get_data_pipeline,
    get_data_pipelines,
)
from .swagger.api.predictions import (
    create_prediction,
    delete_prediction,
    edit_prediction,
    get_prediction,
    get_prediction_forecast,
    get_prediction_result_csv_data,
    get_prediction_result_parquet_data,
    get_predictions,
    save_prediction,
)
from .swagger.api.tests import (
    create_test,
    delete_test,
    edit_test,
    get_test,
    get_test_result_csv_data,
    get_test_result_parquet_data,
    get_tests,
    save_test,
)
from .swagger.types import UNSET
from .utils import (
    check_exists,
    check_not_unset,
    check_response,
    get_data_file,
    wait_for_future_to_succeed,
    wait_for_model_to_complete,
    wait_for_prediction_to_complete,
    wait_for_test_to_complete,
)

ValidDataPrepStepType = Union[
    dto.ApplyFormulaStep,
    dto.CombineStep,
    dto.DropStep,
    dto.FillMissingValueStep,
    dto.FilterRowsStep,
    dto.GroupByStep,
    dto.MergeColumnsStep,
    dto.RenameStep,
    dto.ReplaceStep,
    dto.SentimentAnalysisStep,
    dto.SetColumnLevelStep,
    dto.SetColumnTypeStep,
    dto.SetTimeIndexStep,
    dto.SplitColumnStep,
    dto.TransformTextStep,
]


# pylint: disable=too-many-public-methods
class AnalyzeSwaggerClientWrapper:
    def __init__(self, client: Client):
        self._client = client

    ###############
    # Datasets
    ###############

    def create_file_data_set(
        self,
        data: Union[pd.DataFrame, str, io.TextIOBase, io.BytesIO],
        name: str,
        description: Optional[str] = None,
        parse_options: Optional[dto.ParseOptions] = None,
    ) -> int:
        data_file = get_data_file(data)
        response = create_file_data_set.sync_detailed(
            client=self._client,
            multipart_data=dto.CreateFileDataSetRequest(
                name=name,
                data=data_file,
                description=description or UNSET,
                parse_options=json.dumps(parse_options) if parse_options else UNSET,
            ),
        )
        return wait_for_future_to_succeed(self._client, response).additional_properties[
            "id"
        ]

    def create_cloud_data_set(
        self,
        name: str,
        description: str,
        cloud_type: dto.CreateCloudDataSetRequestCloudType,
        access_key: str,
        secret_key: str,
        bucket_name: str,
        path: str,
        parse_options: Optional[dto.ParseOptions] = None,
    ) -> int:
        payload = dto.CreateCloudDataSetRequest(
            name=name,
            description=description,
            data_origin=dto.CreateCloudDataSetRequestDataOrigin.CLOUD,
            cloud_type=cloud_type,
            access_key=access_key,
            secret_key=secret_key,
            bucket_name=bucket_name,
            path=path,
            parse_options=parse_options or UNSET,
        )
        response = create_cloud_data_set.sync_detailed(
            client=self._client,
            json_body=payload,
        )
        return wait_for_future_to_succeed(self._client, response).additional_properties[
            "id"
        ]

    def create_http_data_set(
        self,
        name: str,
        description: str,
        url: str,
        parse_options: Optional[dto.ParseOptions] = None,
    ) -> int:
        payload = dto.CreateHttpDataSetRequest(
            name=name,
            description=description,
            data_origin=dto.CreateHttpDataSetRequestDataOrigin.HTTP,
            url=url,
            parse_options=parse_options or UNSET,
        )
        response = create_http_data_set.sync_detailed(
            client=self._client,
            json_body=payload,
        )
        return wait_for_future_to_succeed(self._client, response).additional_properties[
            "id"
        ]

    def create_db_data_set(
        self,
        name: str,
        description: str,
        db_type: dto.CreateDbDataSetRequestDbType,
        custom_db_url: bool,
        query: str,
        username: Optional[str] = None,
        password: Optional[str] = None,
        host: Optional[str] = None,
        port: Optional[str] = None,
        db_name: Optional[str] = None,
        db_url: Optional[str] = None,
    ) -> int:
        payload = dto.CreateDbDataSetRequest(
            name=name,
            description=description,
            data_origin=dto.CreateDbDataSetRequestDataOrigin.DB,
            db_type=db_type,
            username=username or UNSET,
            password=password or UNSET,
            host=host or UNSET,
            port=port or UNSET,
            db_name=db_name or UNSET,
            custom_db_url=custom_db_url,
            db_url=db_url or UNSET,
            query=query,
        )
        response = create_db_data_set.sync_detailed(
            client=self._client,
            json_body=payload,
        )
        return wait_for_future_to_succeed(self._client, response).additional_properties[
            "id"
        ]

    def list_data_sets(self) -> List[dto.DataSetResponse]:
        return check_not_unset(
            check_exists(get_data_sets.sync(client=self._client)).data_sets
        )

    def get_data_set(self, data_set_id: int) -> dto.DataSetResponse:
        return check_exists(get_data_set.sync(client=self._client, id=data_set_id))

    def edit_data_set(
        self, data_set_id: int, name: str, description: str = None
    ) -> dto.DataSetResponse:
        payload = dto.EditDataSetRequest(name=name, description=description)
        return check_exists(
            edit_data_set.sync(client=self._client, id=data_set_id, json_body=payload)
        )

    def download_data_set_as_csv(
        self,
        data_set_id: int,
        output_file: Union[str, PathLike],
        separator: str = ",",
        quote_char: str = '"',
        escape_char: str = "\\",
    ) -> None:
        response = get_data_sets_csv_data.sync_detailed(
            client=self._client,
            id=data_set_id,
            separator=separator,
            quote_char=quote_char,
            escape_char=escape_char,
        )
        with open(output_file, "bw") as file:
            file.write(response.content)

    def _get_data_set_bytes_as_parquet(self, data_set_id: int) -> bytes:
        return get_data_sets_parquet_data.sync_detailed(
            client=self._client, id=data_set_id
        ).content

    def download_data_set_as_parquet(
        self, data_set_id: int, output_file: Union[str, PathLike]
    ) -> None:
        with open(output_file, "bw") as file:
            parquet_bytes = self._get_data_set_bytes_as_parquet(data_set_id)
            file.write(parquet_bytes)

    def get_data_set_as_pandas(self, data_set_id: int) -> pd.DataFrame:
        with tempfile.TemporaryFile() as file:
            file.write(self._get_data_set_bytes_as_parquet(data_set_id))
            file.seek(0)
            return pd.read_parquet(path=file)

    def delete_data_set(self, data_set_id: int) -> None:
        delete_data_set.sync(client=self._client, id=data_set_id)

    def apply_saved_data_preparation_steps(
        self,
        data_set_id: int,
        data_preparation_step_id: int,
        insertion_index: Optional[int] = None,
    ) -> int:
        response = apply_data_prep_steps.sync_detailed(
            client=self._client,
            id=data_set_id,
            json_body=dto.ApplyDataPrepStepsRequest(
                saved_steps_id=data_preparation_step_id,
                insertion_index=insertion_index or UNSET,
            ),
        )
        return wait_for_future_to_succeed(self._client, response).additional_properties[
            "id"
        ]

    def apply_new_data_preparation_steps(
        self,
        data_set_id: int,
        steps: List[ValidDataPrepStepType],
        insertion_index: Optional[int] = None,
    ) -> int:
        # Alas, the generated schema doesn't support the oneOf very well,
        # so we accept all the valid steps as inputs and type:ignore the call.
        response = apply_data_prep_steps.sync_detailed(
            client=self._client,
            id=data_set_id,
            json_body=dto.ApplyDataPrepStepsRequest(
                steps=steps,  # type:ignore
                insertion_index=insertion_index or UNSET,
            ),
        )
        return wait_for_future_to_succeed(self._client, response).additional_properties[
            "id"
        ]

    def delete_data_preparation_steps(
        self,
        data_set_id: int,
        delete_index: int,
        delete_children: bool,
    ) -> int:
        response = delete_data_prep_steps.sync_detailed(
            client=self._client,
            id=data_set_id,
            json_body=dto.DeleteDataPrepStepsRequest(
                delete_index=delete_index,
                delete_children=delete_children,
            ),
        )
        return wait_for_future_to_succeed(self._client, response).additional_properties[
            "id"
        ]

    ######################
    # Data set pipelines #
    ######################

    def list_data_pipelines(self) -> List[dto.DataPipelineResponse]:
        return check_not_unset(
            check_exists(get_data_pipelines.sync(client=self._client)).pipelines
        )

    def get_data_pipeline(self, pipeline_id: int) -> dto.DataPipelineResponse:
        return check_exists(get_data_pipeline.sync(client=self._client, id=pipeline_id))

    def delete_data_pipeline(self, pipeline_id: int) -> None:
        delete_data_pipeline.sync(client=self._client, id=pipeline_id)

    def create_data_pipeline(self, data_set_id: int, name: str) -> int:
        payload = dto.DataPipelineCreateRequest(
            data_set_id=data_set_id,
            name=name,
        )
        return check_exists(
            create_data_pipeline.sync(client=self._client, json_body=payload)
        ).id

    def edit_data_pipeline(
        self, pipeline_id: int, name: str
    ) -> dto.DataPipelineResponse:
        payload = dto.EditDataPipelineRequest(name=name)
        return check_exists(
            edit_data_pipeline.sync(
                client=self._client, id=pipeline_id, json_body=payload
            )
        )

    ###############
    # Models
    ###############

    def list_models(self) -> List[dto.ModelResponse]:
        return check_not_unset(
            check_exists(get_models.sync(client=self._client)).models
        )

    def get_model(self, model_id: int) -> dto.ModelResponse:
        return check_exists(get_model.sync(client=self._client, id=model_id))

    def delete_model(self, model_id: int):
        delete_model.sync(client=self._client, id=model_id)

    # pylint: disable=too-many-arguments,too-many-locals
    def create_classification_model(
        self,
        name: str,
        data_set_id: int,
        target_column: str,
        score_to_optimize: dto.ClassificationScorers = dto.ClassificationScorers.F1_WEIGHTED_AVG,
        excluded_columns: Optional[List[str]] = None,
        description: Optional[str] = None,
        draft_mode: bool = False,
        number_of_evaluations: int = 300,
        nlp_language: dto.NlpLanguage = dto.NlpLanguage.AUTO_DETECT,
        model_validation_method: dto.ModelValidationMethod = dto.ModelValidationMethod.FIVE_FOLD_CROSS,
        no_mixing_columns: Optional[List[str]] = None,
        data_partition_method: dto.DataPartitionMethod = dto.DataPartitionMethod.RANDOM,
        partition_column: Optional[str] = None,
        order_by_column: Optional[str] = None,
        wait_until_complete: bool = True,
    ) -> int:
        payload = dto.CreateClassificationRequest(
            name=name,
            data_set_id=data_set_id,
            target_column=target_column,
            score_to_optimize=score_to_optimize,
            model_validation_method=model_validation_method,
            data_partition_method=data_partition_method,
            number_of_evaluations=number_of_evaluations,
            description=description or UNSET,
            excluded_columns=excluded_columns or UNSET,
            nlp_language=nlp_language,
            draft_mode=draft_mode,
            no_mixing_columns=no_mixing_columns or UNSET,
            partition_column=partition_column or UNSET,
            order_by_column=order_by_column or UNSET,
        )

        model_id = check_exists(
            create_classification_model.sync(client=self._client, json_body=payload)
        ).id

        if not wait_until_complete:
            return model_id

        return wait_for_model_to_complete(self._client, model_id)

    # pylint: disable=too-many-arguments,too-many-locals
    def create_regression_model(
        self,
        name: str,
        data_set_id: int,
        target_column: str,
        score_to_optimize: dto.RegressionScorers = dto.RegressionScorers.RMSE,
        excluded_columns: Optional[List[str]] = None,
        description: Optional[str] = None,
        number_of_evaluations: int = 300,
        nlp_language: dto.NlpLanguage = dto.NlpLanguage.AUTO_DETECT,
        draft_mode: bool = False,
        model_validation_method: dto.ModelValidationMethod = dto.ModelValidationMethod.FIVE_FOLD_CROSS,
        no_mixing_columns: Optional[List[str]] = None,
        data_partition_method: dto.DataPartitionMethod = dto.DataPartitionMethod.RANDOM,
        partition_column: Optional[str] = None,
        order_by_column: Optional[str] = None,
        wait_until_complete: bool = True,
    ) -> int:
        payload = dto.CreateRegressionRequest(
            name=name,
            data_set_id=data_set_id,
            target_column=target_column,
            score_to_optimize=score_to_optimize,
            model_validation_method=model_validation_method,
            data_partition_method=data_partition_method,
            number_of_evaluations=number_of_evaluations,
            description=description or UNSET,
            excluded_columns=excluded_columns or UNSET,
            nlp_language=nlp_language,
            draft_mode=draft_mode,
            no_mixing_columns=no_mixing_columns or UNSET,
            partition_column=partition_column or UNSET,
            order_by_column=order_by_column or UNSET,
        )
        model_id = check_exists(
            create_regression_model.sync(client=self._client, json_body=payload)
        ).id
        if not wait_until_complete:
            return model_id

        return wait_for_model_to_complete(self._client, model_id)

    def create_clustering_model(
        self,
        name: str,
        data_set_id: int,
        number_of_clusters: int,
        description: Optional[str] = None,
        nlp_language: dto.NlpLanguage = dto.NlpLanguage.AUTO_DETECT,
        excluded_columns: Optional[List[str]] = None,
        wait_until_complete: bool = True,
    ) -> int:

        payload = dto.CreateClusteringRequest(
            name=name,
            data_set_id=data_set_id,
            description=description or UNSET,
            excluded_columns=excluded_columns or UNSET,
            nlp_language=nlp_language,
            number_of_clusters=number_of_clusters,
        )
        model_id = check_exists(
            create_clustering_model.sync(client=self._client, json_body=payload)
        ).id

        if not wait_until_complete:
            return model_id

        return wait_for_model_to_complete(self._client, model_id)

    def create_autoclustering_model(
        self,
        name: str,
        data_set_id: int,
        minimum_clusters: int,
        maximum_clusters: int,
        description: Optional[str] = None,
        nlp_language: dto.NlpLanguage = dto.NlpLanguage.AUTO_DETECT,
        excluded_columns: Optional[List[str]] = None,
        wait_until_complete: bool = True,
    ) -> int:
        payload = dto.CreateAutoClusteringRequest(
            name=name,
            data_set_id=data_set_id,
            description=description or UNSET,
            excluded_columns=excluded_columns or UNSET,
            nlp_language=nlp_language,
            minimum_clusters=minimum_clusters,
            maximum_clusters=maximum_clusters,
        )
        model_id = check_exists(
            create_auto_clustering_model.sync(client=self._client, json_body=payload)
        ).id

        if not wait_until_complete:
            return model_id

        return wait_for_model_to_complete(self._client, model_id)

    def create_forecasting_model(
        self,
        name: str,
        data_set_id: int,
        target_column: str,
        time_index_column: str,
        cadence: dto.Cadence,
        horizons: List[int],
        description: Optional[str] = None,
        excluded_columns: Optional[List[str]] = None,
        wait_until_complete: bool = True,
    ) -> int:
        payload = dto.CreateForecastingRequest(
            name=name,
            data_set_id=data_set_id,
            description=description or UNSET,
            target_column=target_column,
            time_index_column=time_index_column,
            cadence=cadence,
            horizons=horizons,
            excluded_columns=excluded_columns or UNSET,
        )
        model_id = check_exists(
            create_forecasting_model.sync(client=self._client, json_body=payload)
        ).id

        if not wait_until_complete:
            return model_id

        return wait_for_model_to_complete(self._client, model_id)

    def create_multi_forecasting_model(
        self,
        name: str,
        data_set_id: int,
        target_column: str,
        time_index_column: str,
        cadence: dto.Cadence,
        horizons: List[int],
        discriminator_columns: List[str],
        description: Optional[str] = None,
        excluded_columns: Optional[List[str]] = None,
        wait_until_complete: bool = True,
    ) -> int:
        payload = dto.CreateMultiForecastingRequest(
            name=name,
            data_set_id=data_set_id,
            description=description or UNSET,
            target_column=target_column,
            time_index_column=time_index_column,
            discriminator_columns=discriminator_columns,
            cadence=cadence,
            horizons=horizons,
            excluded_columns=excluded_columns or UNSET,
        )
        model_id = check_exists(
            create_multi_forecasting_model.sync(client=self._client, json_body=payload)
        ).id

        if not wait_until_complete:
            return model_id

        return wait_for_model_to_complete(self._client, model_id)

    def edit_model(
        self, model_id: int, name: str, description: Optional[str] = None
    ) -> dto.ModelResponse:
        payload = dto.EditModelRequest(name=name, description=description)
        return check_exists(
            edit_model.sync(client=self._client, id=model_id, json_body=payload)
        )

    def stop_optimization(self, model_id: int):
        stop_model.sync(client=self._client, id=model_id)

    ###############
    # Prediction
    ###############

    def create_prediction(
        self,
        model_id: int,
        data_set_id: int,
        name: str,
        description: str = None,
        wait_until_complete: bool = True,
    ) -> int:
        prediction_id = check_exists(
            create_prediction.sync(
                client=self._client,
                json_body=dto.CreatePredictionRequest(
                    model_id=model_id,
                    data_set_id=data_set_id,
                    name=name,
                    description=description or UNSET,
                ),
            )
        ).id
        if not wait_until_complete:
            return prediction_id

        return wait_for_prediction_to_complete(self._client, prediction_id)

    def edit_prediction(
        self, prediction_id: int, name: str, description: str = None
    ) -> dto.PredictionResponse:
        payload = dto.EditPredictionRequest(name=name, description=description)
        return check_exists(
            edit_prediction.sync(
                client=self._client, id=prediction_id, json_body=payload
            )
        )

    def get_prediction_overview(self, prediction_id: int) -> dto.PredictionResponse:
        return check_response(
            get_prediction.sync_detailed(client=self._client, id=prediction_id)
        )

    def get_prediction_forecast(
        self, prediction_id: int
    ) -> dto.PredictionForecastResponse:
        return check_response(
            get_prediction_forecast.sync_detailed(client=self._client, id=prediction_id)
        )

    def list_predictions(self) -> List[dto.PredictionResponse]:
        return check_not_unset(
            check_exists(get_predictions.sync(client=self._client)).predictions
        )

    def download_prediction_as_csv(
        self,
        prediction_id: int,
        output_file: Union[str, PathLike],
        separator: str = ",",
        quote_char: str = '"',
        escape_char: str = "\\",
    ) -> None:
        response = get_prediction_result_csv_data.sync_detailed(
            client=self._client,
            id=prediction_id,
            separator=separator,
            quote_char=quote_char,
            escape_char=escape_char,
        )
        with open(output_file, "bw") as file:
            file.write(response.content)

    def _get_prediction_bytes_as_parquet(self, prediction_id: int) -> bytes:
        return get_prediction_result_parquet_data.sync_detailed(
            client=self._client, id=prediction_id
        ).content

    def download_prediction_as_parquet(
        self, prediction_id: int, output_file: Union[str, PathLike]
    ) -> None:
        with open(output_file, "bw") as file:
            parquet_bytes = self._get_prediction_bytes_as_parquet(prediction_id)
            file.write(parquet_bytes)

    def get_prediction_result_as_pandas(self, prediction_id: int) -> pd.DataFrame:
        with tempfile.TemporaryFile() as file:
            file.write(self._get_prediction_bytes_as_parquet(prediction_id))
            file.seek(0)
            return pd.read_parquet(path=file)

    def delete_prediction(self, prediction_id: int) -> None:
        delete_prediction.sync(client=self._client, id=prediction_id)

    def save_prediction_result_as_data_set(
        self, prediction_id: int, name: str, description: Optional[str] = None
    ) -> int:
        payload = dto.SavePredictionRequest(name=name, description=description)
        return check_exists(
            save_prediction.sync(
                client=self._client, id=prediction_id, json_body=payload
            )
        ).id

    ###############
    # Tests
    ###############

    def create_test(
        self,
        model_id: int,
        data_set_id: int,
        name: str,
        description: str = None,
        wait_until_complete: bool = True,
    ) -> int:
        test_id = check_exists(
            create_test.sync(
                client=self._client,
                json_body=dto.CreateTestRequest(
                    model_id=model_id,
                    data_set_id=data_set_id,
                    name=name,
                    description=description or UNSET,
                ),
            )
        ).id
        if not wait_until_complete:
            return test_id

        return wait_for_test_to_complete(self._client, test_id)

    def edit_test(
        self, test_id: int, name: str, description: str = None
    ) -> dto.TestResponse:
        payload = dto.EditTestRequest(name=name, description=description)
        return check_exists(
            edit_test.sync(client=self._client, id=test_id, json_body=payload)
        )

    def get_test_overview(self, test_id: int) -> dto.TestResponse:
        return check_response(get_test.sync_detailed(client=self._client, id=test_id))

    def list_test(self) -> List[dto.TestResponse]:
        return check_not_unset(check_exists(get_tests.sync(client=self._client)).tests)

    def download_test_as_csv(
        self,
        test_id: int,
        output_file: Union[str, PathLike],
        separator: str = ",",
        quote_char: str = '"',
        escape_char: str = "\\",
    ) -> None:
        response = get_test_result_csv_data.sync_detailed(
            client=self._client,
            id=test_id,
            separator=separator,
            quote_char=quote_char,
            escape_char=escape_char,
        )
        with open(output_file, "bw") as file:
            file.write(response.content)

    def _get_test_bytes_as_parquet(self, test_id: int) -> bytes:
        return get_test_result_parquet_data.sync_detailed(
            client=self._client, id=test_id
        ).content

    def download_test_as_parquet(
        self, test_id: int, output_file: Union[str, PathLike]
    ) -> None:
        with open(output_file, "bw") as file:
            parquet_bytes = self._get_test_bytes_as_parquet(test_id)
            file.write(parquet_bytes)

    def get_test_result_as_pandas(self, test_id: int) -> pd.DataFrame:
        with tempfile.TemporaryFile() as file:
            file.write(self._get_test_bytes_as_parquet(test_id))
            file.seek(0)
            return pd.read_parquet(path=file)

    def delete_test(self, test_id: int) -> None:
        delete_test.sync(client=self._client, id=test_id)

    def save_test_result_as_data_set(
        self, test_id: int, name: str, description: Optional[str] = None
    ) -> int:
        payload = dto.SaveTestRequest(name=name, description=description)
        return check_exists(
            save_test.sync(client=self._client, id=test_id, json_body=payload)
        ).id

    ################
    # APIs - Model #
    ################

    def list_model_apis(self) -> List[dto.DeployedModelApiResponse]:
        return check_not_unset(
            check_exists(get_model_apis.sync(client=self._client)).published_models
        )

    def delete_model_api(self, model_api_id: int) -> None:
        delete_model_api.sync(client=self._client, model_api_id=model_api_id)

    def create_model_api(
        self, model_id: int, name: str, description: Optional[str] = None
    ) -> int:
        response = create_model_api.sync_detailed(
            client=self._client,
            json_body=dto.CreateDeployedModelApiRequest(
                model_id=model_id, name=name, description=description or UNSET
            ),
        )
        return wait_for_future_to_succeed(self._client, response).additional_properties[
            "id"
        ]

    def edit_model_api(
        self, api_id: int, name: str, description: Optional[str] = None
    ) -> dto.DeployedModelApiResponse:
        payload = dto.EditDeployedModelApiRequest(name=name, description=description)
        return check_exists(
            edit_model_api.sync(
                client=self._client, model_api_id=api_id, json_body=payload
            )
        )

    def get_model_api(self, api_id: int) -> dto.DeployedModelApiResponse:
        return check_exists(
            get_model_api.sync(client=self._client, model_api_id=api_id)
        )

    def list_model_api_clients(
        self, api_id: int
    ) -> List[dto.DeployedApiClientResponse]:
        return check_not_unset(
            check_exists(
                get_model_api_clients.sync(client=self._client, model_api_id=api_id)
            ).api_clients
        )

    def delete_model_api_client(self, model_api_id: int, client_id: str) -> None:
        delete_model_api_client.sync(
            client=self._client, model_api_id=model_api_id, client_id=client_id
        )

    def create_model_api_client(
        self, api_id: int, name: str
    ) -> dto.DeployedApiClientCreateResponse:
        return check_exists(
            create_model_api_client.sync(
                client=self._client,
                model_api_id=api_id,
                json_body=dto.CreateDeployedApiClientRequest(name=name),
            )
        )

    def get_model_api_client(
        self, api_id: int, client_id: str
    ) -> dto.DeployedApiClientResponse:
        return check_exists(
            get_model_api_client.sync(
                client=self._client, model_api_id=api_id, client_id=client_id
            )
        )

    ###################
    # APIs - Pipeline #
    ###################

    def list_pipeline_apis(self) -> List[dto.DeployedPipelineApiResponse]:
        return check_not_unset(
            check_exists(
                get_pipeline_apis.sync(client=self._client)
            ).published_pipelines
        )

    def delete_pipeline_api(self, pipeline_api_id: int) -> None:
        delete_pipeline_api.sync(client=self._client, pipeline_api_id=pipeline_api_id)

    def create_pipeline_api(
        self, pipeline_id: int, name: str, description: Optional[str] = None
    ) -> int:
        response = create_pipeline_api.sync_detailed(
            client=self._client,
            json_body=dto.CreateDeployedPipelineApiRequest(
                pipeline_id=pipeline_id, name=name, description=description or UNSET
            ),
        )
        return wait_for_future_to_succeed(self._client, response).additional_properties[
            "id"
        ]

    def edit_pipeline_api(
        self, api_id: int, name: str, description: Optional[str] = None
    ) -> dto.DeployedPipelineApiResponse:
        payload = dto.EditDeployedPipelineApiRequest(name=name, description=description)
        return check_exists(
            edit_pipeline_api.sync(
                client=self._client, pipeline_api_id=api_id, json_body=payload
            )
        )

    def get_pipeline_api(self, api_id: int) -> dto.DeployedPipelineApiResponse:
        return check_exists(
            get_pipeline_api.sync(client=self._client, pipeline_api_id=api_id)
        )

    def list_pipeline_api_clients(
        self, api_id: int
    ) -> List[dto.DeployedApiClientResponse]:
        return check_not_unset(
            check_exists(
                get_pipeline_api_clients.sync(
                    client=self._client, pipeline_api_id=api_id
                )
            ).api_clients
        )

    def delete_pipeline_api_client(self, pipeline_api_id: int, client_id: str) -> None:
        delete_pipeline_api_client.sync(
            client=self._client, pipeline_api_id=pipeline_api_id, client_id=client_id
        )

    def create_pipeline_api_client(
        self, api_id: int, name: str
    ) -> dto.DeployedApiClientCreateResponse:
        return check_exists(
            create_pipeline_api_client.sync(
                client=self._client,
                pipeline_api_id=api_id,
                json_body=dto.CreateDeployedApiClientRequest(name=name),
            )
        )

    def get_pipeline_api_client(
        self, api_id: int, client_id: str
    ) -> dto.DeployedApiClientResponse:
        return check_exists(
            get_pipeline_api_client.sync(
                client=self._client, pipeline_api_id=api_id, client_id=client_id
            )
        )



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.data_value import DataValue
from ..types import UNSET, Unset

T = TypeVar("T", bound="DiscriminatorsColumnValuesItem")

@attr.s(auto_attribs=True)
class DiscriminatorsColumnValuesItem:
    """  """
    column_name: Union[Unset, str] = UNSET
    value: Union[DataValue, Unset] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        column_name =  self.column_name
        value: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.value, Unset):
            value = self.value.to_dict()


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if column_name is not UNSET:
            field_dict["columnName"] = column_name
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        column_name = d.pop("columnName", UNSET)

        value: Union[DataValue, Unset] = UNSET
        _value = d.pop("value", UNSET)
        if not isinstance(_value,  Unset):
            value = DataValue.from_dict(_value)




        discriminators_column_values_item = cls(
            column_name=column_name,
            value=value,
        )

        discriminators_column_values_item.additional_properties = d
        return discriminators_column_values_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TestResponseProblemType(str, Enum):
    CLASSIFICATION = "classification"
    CLUSTERING = "clustering"
    REGRESSION = "regression"
    FORECASTING = "forecasting"
    UNKNOWN = "unknown"

    def __str__(self) -> str:
        return str(self.value)


from enum import Enum


class ScoreType(str, Enum):
    UNKNOWN = "UNKNOWN"
    F1_WEIGHTED_AVG = "F1_WEIGHTED_AVG"
    ACCURACY = "ACCURACY"
    ROC_AUC = "ROC_AUC"
    RMSE = "RMSE"
    EXPLAINED_VARIANCE = "EXPLAINED_VARIANCE"
    PREDICT_TIME = "PREDICT_TIME"
    TEST_TIME = "TEST_TIME"
    TRAIN_TIME = "TRAIN_TIME"
    PRECISION = "PRECISION"
    RECALL = "RECALL"
    ABSOLUTE_ERROR = "ABSOLUTE_ERROR"
    R2 = "R2"
    MEDIAN_ABSOLUTE_ERROR = "MEDIAN_ABSOLUTE_ERROR"
    LOG_LIKELIHOOD = "LOG_LIKELIHOOD"
    PERCENT_ERROR = "PERCENT_ERROR"
    BIC = "BIC"
    ROI = "ROI"
    CALINSKI_HARABASZ = "CALINSKI_HARABASZ"
    CLUSTERS = "CLUSTERS"
    SILHOUETTE_COEF = "SILHOUETTE_COEF"
    F1_SINGLE_CLASS = "F1_SINGLE_CLASS"
    RECALL_SINGLE_CLASS = "RECALL_SINGLE_CLASS"
    PRECISION_SINGLE_CLASS = "PRECISION_SINGLE_CLASS"
    F5_SINGLE_CLASS = "F5_SINGLE_CLASS"

    def __str__(self) -> str:
        return str(self.value)


from typing import List, Optional

from ....swagger.models import ModelResponse, ModelStatus
from ....utils import CachedModelResponse, wait_for_model_to_complete
from ....utils.typing import PathLike
from ...connection import (
    ConnectionLike,
    client_from_connectionlike,
    from_connectionlike,
)
from ...data_set import DataLike, get_dataset_from_datalike
from ...prediction import Prediction
from ...test import Test
from ..serialization import SerializedModel, save_model
from .base import BaseModel


class RemoteModel(BaseModel):
    """
    A model that is saved to Analyze
    """

    def __init__(self, connection: ConnectionLike, model_id: int):
        self._model_id = model_id
        self._connection = from_connectionlike(connection)
        self._client = client_from_connectionlike(connection)
        self._cached_model_info = CachedModelResponse(self._client, self.model_id)

    @classmethod
    def from_serialized_model(cls, connection, serialized_model) -> "RemoteModel":
        if serialized_model.base_url != connection.base_url:
            raise Exception(
                "The connection and the model are from different instances of Analyze"
            )

        remote_model = RemoteModel(connection, serialized_model.model_id)

        if remote_model.model_info().project_id != serialized_model.project_id:
            raise Exception(
                "The connection and the model are from different projects within Analyze"
            )
        return remote_model

    @property
    def model_id(self) -> int:
        """The ID of the model in Analyze"""
        return self._model_id

    def is_fitting(self) -> bool:
        # Any remote model counts as 'fitting' for our purposes unless the computation has completed
        # Later we could split out the model having been saved from the model post-calculation
        return self.model_info().status not in (
            ModelStatus.COMPLETE,
            ModelStatus.FAILED,
        )

    def is_fitted(self) -> bool:
        return self.model_info().status == ModelStatus.COMPLETE

    def has_failed_fitting(self) -> bool:
        return self.model_info().status == ModelStatus.FAILED

    def wait_until_fitted(self):
        wait_for_model_to_complete(self._client, self.model_id)

    def save(self, path: PathLike) -> None:
        save_model(
            path,
            SerializedModel(
                base_url=self._connection.base_url,
                project_id=self.model_info().project_id,
                model_id=self.model_id,
                problem_type=self.model_info().problem_type,
            ),
        )

    def model_info(self) -> ModelResponse:
        """Get information about the model"""
        # For the moment, the model information will just be the model response object.
        return self._cached_model_info.get_response()

    def url(self) -> str:
        base_url = self._connection.base_url
        project_id = self.model_info().project_id
        model_id = self.model_id

        return f"{base_url}/{project_id}/models/{model_id}"

    def predict(
        self,
        data: DataLike,
        *,
        name: str,
        description: Optional[str],
        wait_until_complete: bool,
    ) -> Prediction:
        if not self.is_fitted():
            raise Exception("Model must be fitted before performing a prediction")
        data = get_dataset_from_datalike(data, connection=self._connection)
        return Prediction.create_prediction(
            self.model_id,
            data.data_set_id,
            name=name,
            description=description,
            connection=self._connection,
            wait_until_complete=wait_until_complete,
        )

    def test(
        self,
        data: DataLike,
        *,
        name: Optional[str] = None,
        description: Optional[str] = None,
        wait_until_complete: bool = True,
    ) -> Test:
        if not self.is_fitted():
            raise Exception("Model must be fitted before performing a test")
        data = get_dataset_from_datalike(data, connection=self._connection)
        return Test.create_test(
            self.model_id,
            data.data_set_id,
            name=name,
            description=description,
            connection=self._connection,
            wait_until_complete=wait_until_complete,
        )

    ######################
    # Local only methods #
    ######################

    def fit(
        self,
        data: DataLike,
        target: str,
        excluded_columns: Optional[List[str]] = None,
        *,
        order_by: Optional[str] = None,
        partition_by: Optional[str] = None,
        no_mixing: Optional[List[str]] = None,
    ) -> BaseModel:
        raise Exception("Model is already fitted - cannot refit")



from enum import Enum


class GroupByAggregationRule(str, Enum):
    SUM = "sum"
    MEAN = "mean"
    STD_DEV = "std_dev"
    MODE = "mode"
    MIN = "min"
    MAX = "max"
    RANGE = "range"
    COUNT_ALL = "count_all"
    COUNT_NON_NULL = "count_non_null"
    COUNT_NON_NULL_DISTINCT = "count_non_null_distinct"

    def __str__(self) -> str:
        return str(self.value)


from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.discriminators_column_values_item import DiscriminatorsColumnValuesItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="Discriminators")

@attr.s(auto_attribs=True)
class Discriminators:
    """  """
    empty: bool
    column_values: Union[Unset, List[DiscriminatorsColumnValuesItem]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        empty =  self.empty
        column_values: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.column_values, Unset):
            column_values = []
            for column_values_item_data in self.column_values:
                column_values_item = column_values_item_data.to_dict()

                column_values.append(column_values_item)





        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "empty": empty,
        })
        if column_values is not UNSET:
            field_dict["columnValues"] = column_values

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        empty = d.pop("empty")

        column_values = []
        _column_values = d.pop("columnValues", UNSET)
        for column_values_item_data in (_column_values or []):
            column_values_item = DiscriminatorsColumnValuesItem.from_dict(column_values_item_data)



            column_values.append(column_values_item)


        discriminators = cls(
            empty=empty,
            column_values=column_values,
        )

        discriminators.additional_properties = d
        return discriminators

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import io
import json
from typing import Any, Dict

import attr

from ...swagger.models import ProblemType
from ...utils.typing import PathLike


@attr.s(auto_attribs=True)
class SerializedModel:
    base_url: str
    model_id: int
    project_id: int
    problem_type: ProblemType


def parse_model_from_path(path: PathLike):
    with open(path, mode="r") as file:
        config = json.load(file)
    return parse_model(config)


def parse_model(config: Dict[str, Any]) -> SerializedModel:
    if "model" not in config.keys():
        raise Exception("Not a model")
    model_config = config["model"]

    if "base_url" not in model_config.keys():
        raise Exception(
            "Configuration did not represent an Analyze model - No base_url saved"
        )
    base_url = model_config["base_url"]

    if "project_id" not in model_config.keys():
        raise Exception(
            "Configuration did not represent an Analyze model - No project_id saved"
        )
    project_id = model_config["project_id"]

    if "model_id" not in model_config.keys():
        raise Exception(
            "Configuration did not represent an Analyze model - No model_id saved"
        )
    model_id = model_config["model_id"]

    if "problem_type" not in model_config.keys():
        raise Exception(
            "Configuration did not represent an Analyze model - No problem_type saved"
        )
    try:
        problem_type = ProblemType[model_config["problem_type"]]
    except KeyError:
        raise Exception(  # pylint: disable=raise-missing-from
            "Configuration did not represent an Analyze model - problem_type did not match a valid Analyze problem type"
        )

    return SerializedModel(
        base_url=base_url,
        project_id=project_id,
        model_id=model_id,
        problem_type=problem_type,
    )


def save_model(path: PathLike, serialized_model: SerializedModel):
    with io.open(path, mode="w") as file:
        dict_repr = {
            "model": {
                "base_url": serialized_model.base_url,
                "project_id": serialized_model.project_id,
                "model_id": serialized_model.model_id,
                "problem_type": str(serialized_model.problem_type),
            }
        }
        json.dump(dict_repr, file)



from typing import Any, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..models.model_score import ModelScore
from ..models.status import Status
from ..models.test_response_problem_type import TestResponseProblemType
from ..models.test_response_task_status import TestResponseTaskStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="TestResponse")

@attr.s(auto_attribs=True)
class TestResponse:
    """  """
    id: int
    name: str
    problem_type: TestResponseProblemType
    task_status: TestResponseTaskStatus
    description: Union[Unset, Optional[str]] = UNSET
    project_id: Union[Unset, Optional[int]] = UNSET
    model_id: Union[Unset, Optional[int]] = UNSET
    data_set_id: Union[Unset, Optional[int]] = UNSET
    status: Union[Unset, Status] = UNSET
    failure_details: Union[Unset, Optional[str]] = UNSET
    target_column_name: Union[Unset, Optional[str]] = UNSET
    predicted_column_name: Union[Unset, Optional[str]] = UNSET
    score: Union[Optional[ModelScore], Unset] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        id =  self.id
        name =  self.name
        problem_type = self.problem_type.value

        task_status = self.task_status.value

        description =  self.description
        project_id =  self.project_id
        model_id =  self.model_id
        data_set_id =  self.data_set_id
        status: Union[Unset, Status] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status

        failure_details =  self.failure_details
        target_column_name =  self.target_column_name
        predicted_column_name =  self.predicted_column_name
        score: Union[None, Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.score, Unset):
            score = self.score.to_dict() if self.score else None


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "name": name,
            "problemType": problem_type,
            "taskStatus": task_status,
        })
        if description is not UNSET:
            field_dict["description"] = description
        if project_id is not UNSET:
            field_dict["projectId"] = project_id
        if model_id is not UNSET:
            field_dict["modelId"] = model_id
        if data_set_id is not UNSET:
            field_dict["dataSetId"] = data_set_id
        if status is not UNSET:
            field_dict["status"] = status
        if failure_details is not UNSET:
            field_dict["failureDetails"] = failure_details
        if target_column_name is not UNSET:
            field_dict["targetColumnName"] = target_column_name
        if predicted_column_name is not UNSET:
            field_dict["predictedColumnName"] = predicted_column_name
        if score is not UNSET:
            field_dict["score"] = score

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        problem_type = TestResponseProblemType(d.pop("problemType"))




        task_status = TestResponseTaskStatus(d.pop("taskStatus"))




        description = d.pop("description", UNSET)

        project_id = d.pop("projectId", UNSET)

        model_id = d.pop("modelId", UNSET)

        data_set_id = d.pop("dataSetId", UNSET)

        status: Union[Unset, Status] = UNSET
        _status = d.pop("status", UNSET)
        if not isinstance(_status,  Unset):
            status = Status(_status)




        failure_details = d.pop("failureDetails", UNSET)

        target_column_name = d.pop("targetColumnName", UNSET)

        predicted_column_name = d.pop("predictedColumnName", UNSET)

        score = None
        _score = d.pop("score", UNSET)
        if _score is not None and not isinstance(_score,  Unset):
            score = ModelScore.from_dict(_score)




        test_response = cls(
            id=id,
            name=name,
            problem_type=problem_type,
            task_status=task_status,
            description=description,
            project_id=project_id,
            model_id=model_id,
            data_set_id=data_set_id,
            status=status,
            failure_details=failure_details,
            target_column_name=target_column_name,
            predicted_column_name=predicted_column_name,
            score=score,
        )

        test_response.additional_properties = d
        return test_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.nlp_language import NlpLanguage
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateClusteringRequest")

@attr.s(auto_attribs=True)
class CreateClusteringRequest:
    """  """
    name: str
    data_set_id: int
    number_of_clusters: int
    description: Union[Unset, str] = UNSET
    excluded_columns: Union[Unset, List[str]] = UNSET
    nlp_language: Union[Unset, NlpLanguage] = NlpLanguage.AUTO_DETECT
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        name =  self.name
        data_set_id =  self.data_set_id
        number_of_clusters =  self.number_of_clusters
        description =  self.description
        excluded_columns: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.excluded_columns, Unset):
            excluded_columns = self.excluded_columns




        nlp_language: Union[Unset, NlpLanguage] = UNSET
        if not isinstance(self.nlp_language, Unset):
            nlp_language = self.nlp_language


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "name": name,
            "dataSetId": data_set_id,
            "numberOfClusters": number_of_clusters,
        })
        if description is not UNSET:
            field_dict["description"] = description
        if excluded_columns is not UNSET:
            field_dict["excludedColumns"] = excluded_columns
        if nlp_language is not UNSET:
            field_dict["nlpLanguage"] = nlp_language

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        data_set_id = d.pop("dataSetId")

        number_of_clusters = d.pop("numberOfClusters")

        description = d.pop("description", UNSET)

        excluded_columns = cast(List[str], d.pop("excludedColumns", UNSET))


        nlp_language: Union[Unset, NlpLanguage] = UNSET
        _nlp_language = d.pop("nlpLanguage", UNSET)
        if not isinstance(_nlp_language,  Unset):
            nlp_language = NlpLanguage(_nlp_language)




        create_clustering_request = cls(
            name=name,
            data_set_id=data_set_id,
            number_of_clusters=number_of_clusters,
            description=description,
            excluded_columns=excluded_columns,
            nlp_language=nlp_language,
        )

        create_clustering_request.additional_properties = d
        return create_clustering_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.prediction_forecast_response_model_forecasts_item import (
    PredictionForecastResponseModelForecastsItem,
)

T = TypeVar("T", bound="PredictionForecastResponse")

@attr.s(auto_attribs=True)
class PredictionForecastResponse:
    """  """
    id: int
    model_forecasts: List[PredictionForecastResponseModelForecastsItem]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        id =  self.id
        model_forecasts = []
        for model_forecasts_item_data in self.model_forecasts:
            model_forecasts_item = model_forecasts_item_data.to_dict()

            model_forecasts.append(model_forecasts_item)





        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "modelForecasts": model_forecasts,
        })

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        model_forecasts = []
        _model_forecasts = d.pop("modelForecasts")
        for model_forecasts_item_data in (_model_forecasts):
            model_forecasts_item = PredictionForecastResponseModelForecastsItem.from_dict(model_forecasts_item_data)



            model_forecasts.append(model_forecasts_item)


        prediction_forecast_response = cls(
            id=id,
            model_forecasts=model_forecasts,
        )

        prediction_forecast_response.additional_properties = d
        return prediction_forecast_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import tempfile
from datetime import datetime
from enum import Enum
from io import BytesIO
from typing import IO, Dict, List, Optional, Union

import pandas as pd

from ..swagger import Client
from ..swagger.api.data import (
    apply_data_prep_steps,
    create_file_data_set,
    get_data_set,
    get_data_sets_parquet_data,
)
from ..swagger.models import (
    ApplyDataPrepStepsRequest,
    CreateFileDataSetRequest,
    DataPrepStepResponse,
    DataSetResponse,
    SetColumnTypeStep,
    SetColumnTypeStepStepType,
)
from ..swagger.types import UNSET, File
from ..utils import wait_for_future_to_succeed
from .connection import ConnectionLike, client_from_connectionlike, from_connectionlike


def _default_dataset_name() -> str:
    return (
        f"Data set from Python Client {datetime.utcnow().strftime('%Y/%m/%d %H:%M:%S')}"
    )


def _upload_file(
    client: Client,
    name: str,
    description: Optional[str],
    file: Union[IO[str], IO[bytes]],
) -> int:
    payload = CreateFileDataSetRequest(
        name=name,
        data=File(file_name="input", payload=file),  # type: ignore
        description=description or UNSET,
    )
    future = create_file_data_set.sync_detailed(
        client=client,
        multipart_data=payload,
    )
    future_response = wait_for_future_to_succeed(client, future)
    return future_response.additional_properties["id"]


def _download_as_parquet(
    client: Client,
    data_set_id: int,
) -> BytesIO:
    parquet_bytes = get_data_sets_parquet_data.sync_detailed(
        client=client, id=data_set_id
    ).content
    return BytesIO(parquet_bytes)


class ColumnType(str, Enum):
    INT = "INT"
    FLOAT = "FLOAT"
    TEXT = "TEXT"
    DATETIME = "DATETIME"


def _set_type_step(
    column_names: List[str], type_: ColumnType
) -> Optional[SetColumnTypeStep]:
    if len(column_names) == 0:
        return None
    return SetColumnTypeStep(
        columns=column_names,
        new_type=type_.name,
        step_type=SetColumnTypeStepStepType.TYPE,
        invalid="null",
        not_int="null",
        # Use some sensible defaults for these
        # Could potentially allow the user to specify them?
        epoch="unix_secs",
        datetime_format="%Y/%m/%d %H:%M:%S",
    )


def _apply_steps(client: Client, data_set_id: int, steps):
    response = apply_data_prep_steps.sync_detailed(
        client=client,
        id=data_set_id,
        json_body=ApplyDataPrepStepsRequest(
            steps=steps,  # type:ignore
            insertion_index=UNSET,
        ),
    )
    return wait_for_future_to_succeed(client, response).additional_properties["id"]


def _pretty_step(step_number: int, step: DataPrepStepResponse) -> str:
    return "# {step_number}: '{step_type}' - {status} {warning_count} warning(s)".format(
        step_number=step_number,
        step_type=step.step.additional_properties["stepType"],
        # TODO: Pretty print all the additional properties ^
        status=step.status,
        warning_count=step.warning_count,
    )


def _pretty_info(info: DataSetResponse) -> str:
    inner_steps_text = "\n".join(
        [
            _pretty_step(step_number, step)
            for step_number, step in enumerate(info.data_prep_steps)
        ]
    )
    if inner_steps_text == "":
        steps_text = "# Steps: None"
    else:
        steps_text = "# Steps\n" + inner_steps_text

    return """\
########################################
# Analyze Data Set
# ------------------------------------
# Project: {project_id}
# Data Set Id: {id}
# Name: {name}
# Description: {description}
# Shape: ({columns} columns x {rows} rows)
# Created at {created_at} by {created_by}
# Status: {status}
{steps}
########################################
""".format(
        project_id=info.project_id,
        id=info.id,
        name=info.name,
        description=info.description,
        columns=info.columns,
        rows=info.rows,
        created_at=info.created_at,
        created_by=info.created_by,
        status=info.status,
        steps=steps_text,
    )


class DataSet:
    """
    An analyze data set stored on the system
    """

    def __init__(self, data_set_id: int, *, connection: ConnectionLike):
        self._connection = from_connectionlike(connection)
        self._client = client_from_connectionlike(connection)
        self.data_set_id = data_set_id

    def as_df(self) -> pd.DataFrame:
        return pd.read_parquet(_download_as_parquet(self._client, self.data_set_id))

    def set_type(self, types: Dict[str, ColumnType]):
        steps = [_set_type_step([column], type_) for column, type_ in types.items()]
        steps = [step for step in steps if step is not None]
        _apply_steps(self._client, self.data_set_id, steps)

    @property
    def info(self) -> DataSetResponse:
        # We could cache this once it is succeeded / failed *if* we
        # (a) cleared it each time we ran steps, and
        # (b) didn't mind if it was out of date if the UI was used to run a data prep step
        response = get_data_set.sync(client=self._client, id=self.data_set_id)
        if response is None:
            raise Exception("Failed to get data set information from server")
        return response

    @property
    def pretty_info(self) -> str:
        return _pretty_info(self.info)

    @property
    def url(self) -> str:
        base_url = self._connection.base_url
        project_id = self.info.project_id
        data_set_id = self.data_set_id
        return f"{base_url}/{project_id}/data/{data_set_id}"

    @classmethod
    def from_df(
        cls,
        data: pd.DataFrame,
        name: Optional[str] = None,
        description: Optional[str] = None,
        *,
        connection: ConnectionLike,
    ) -> "DataSet":
        client = client_from_connectionlike(connection)
        name = name or _default_dataset_name()
        with tempfile.TemporaryFile(mode="w+") as file:
            data.to_csv(file, index=False)
            file.seek(0)
            data_set_id = _upload_file(client, name, description, file)

        return DataSet(data_set_id, connection=connection)

    @classmethod
    def from_csv(
        cls,
        file_name: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
        *,
        connection: ConnectionLike,
    ) -> "DataSet":
        client = client_from_connectionlike(connection)
        name = name or _default_dataset_name()
        with open(file_name, "rb") as file:
            data_set_id = _upload_file(client, name, description, file)
        return DataSet(data_set_id, connection=connection)

    @classmethod
    def from_server(cls, data_set_id: int, *, connection: ConnectionLike) -> "DataSet":
        return DataSet(data_set_id, connection=connection)


DataLike = Union[pd.DataFrame, DataSet]


def get_dataset_from_datalike(data: DataLike, *, connection: ConnectionLike) -> DataSet:
    if isinstance(data, DataSet):
        return data
    return DataSet.from_df(data, connection=connection)


def load_data_set_from_server(
    data_set_id: int, *, connection: ConnectionLike
) -> DataSet:
    return DataSet.from_server(data_set_id, connection=connection)


def from_df(
    data: pd.DataFrame,
    name: Optional[str] = None,
    description: Optional[str] = None,
    *,
    connection: ConnectionLike,
) -> DataSet:
    return DataSet.from_df(data, name, description, connection=connection)


def from_csv(
    file_name: str,
    name: Optional[str] = None,
    description: Optional[str] = None,
    *,
    connection: ConnectionLike,
) -> DataSet:
    return DataSet.from_csv(file_name, name, description, connection=connection)


__all__ = [
    "ColumnType",
    "DataLike",
    "DataSet",
    "load_data_set_from_server",
    "from_df",
    "from_csv",
]



from ._cached_response import (
    CachedModelResponse,
    CachedPredictionResponse,
    CachedTestResponse,
)
from ._polling import (
    wait_for_future_to_succeed,
    wait_for_model_to_complete,
    wait_for_prediction_to_complete,
    wait_for_test_to_complete,
)
from ._refreshable_access_token import RefreshableAccessToken
from ._swagger_response_utils import (
    check_exists,
    check_not_unset,
    check_response,
    get_data_file,
)
from ._token_authenticated_client import TokenAuthenticatedClient



from enum import Enum


class TestResponseTaskStatus(str, Enum):
    PENDING = "pending"
    STARTING = "starting"
    SCHEDULED = "scheduled"
    RUNNING = "running"
    DONE = "done"
    FAILED = "failed"

    def __str__(self) -> str:
        return str(self.value)


from typing import Any, Dict, List, Type, TypeVar, cast

import attr

from ..models.group_by_aggregation import GroupByAggregation
from ..models.group_by_step_step_type import GroupByStepStepType

T = TypeVar("T", bound="GroupByStep")

@attr.s(auto_attribs=True)
class GroupByStep:
    """  """
    key_columns: List[str]
    aggregations: List[GroupByAggregation]
    step_type: GroupByStepStepType
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        key_columns = self.key_columns




        aggregations = []
        for aggregations_item_data in self.aggregations:
            aggregations_item = aggregations_item_data.to_dict()

            aggregations.append(aggregations_item)




        step_type = self.step_type.value


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "keyColumns": key_columns,
            "aggregations": aggregations,
            "stepType": step_type,
        })

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        key_columns = cast(List[str], d.pop("keyColumns"))


        aggregations = []
        _aggregations = d.pop("aggregations")
        for aggregations_item_data in (_aggregations):
            aggregations_item = GroupByAggregation.from_dict(aggregations_item_data)



            aggregations.append(aggregations_item)


        step_type = GroupByStepStepType(d.pop("stepType"))




        group_by_step = cls(
            key_columns=key_columns,
            aggregations=aggregations,
            step_type=step_type,
        )

        group_by_step.additional_properties = d
        return group_by_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CreateDbDataSetRequestDbType(str, Enum):
    POSTGRESQL = "postgresql"
    MYSQL = "mysql"
    D_B2 = "db2"
    SQLSERVER = "sqlserver"

    def __str__(self) -> str:
        return str(self.value)


from datetime import datetime
from io import BytesIO
from typing import Optional, Union

import pandas as pd

from ..swagger import Client
from ..swagger.api.predictions import (
    create_prediction,
    get_prediction_result_parquet_data,
    save_prediction,
)
from ..swagger.models import (
    CreatePredictionRequest,
    PredictionResponse,
    PredictionResponseProblemType,
    PredictionResponseTaskStatus,
    SavePredictionRequest,
    Status,
)
from ..swagger.types import UNSET, Unset
from ..utils import (
    CachedPredictionResponse,
    check_exists,
    wait_for_prediction_to_complete,
)
from .connection import ConnectionLike, client_from_connectionlike, from_connectionlike
from .data_set import DataSet, load_data_set_from_server


def _default_saved_dataset_name() -> str:
    return f"Saved Prediction from Python Client {datetime.utcnow().strftime('%Y/%m/%d %H:%M:%S')}"


def _download_as_parquet(
    client: Client,
    prediction_id: int,
) -> BytesIO:
    parquet_bytes = get_prediction_result_parquet_data.sync_detailed(
        client=client, id=prediction_id
    ).content
    return BytesIO(parquet_bytes)


def _raise_if_prediction_is_not_finished(status: Union[Unset, Status]):
    if isinstance(status, Unset):
        raise Exception("Status could not be obtained")

    if status == Status.SUCCEEDED:
        return

    if status == Status.IN_PROGRESS:
        raise Exception("The prediction is still running")
    if status == Status.FAILED:
        raise Exception("The prediction has failed")
    raise Exception("The prediction has not succeeded")


class Prediction:
    def __init__(self, prediction_id: int, *, connection: ConnectionLike):
        self.prediction_id = prediction_id
        self._connection = from_connectionlike(connection)
        self._client = client_from_connectionlike(connection)
        self._cached_prediction_info = CachedPredictionResponse(
            self._client, self.prediction_id
        )

    @classmethod
    def create_prediction(
        cls,
        model_id: int,
        data_set_id: int,
        *,
        connection: ConnectionLike,
        name: str,
        description: Optional[str],
        wait_until_complete: bool,
    ) -> "Prediction":
        """
        Create a new prediction for the specified Analyze model id and data set id.
        Usually you should use the `model.predict(data_set)` method rather than this method.
        """
        client = client_from_connectionlike(connection)
        prediction_id = check_exists(
            create_prediction.sync(
                client=client,
                json_body=CreatePredictionRequest(
                    model_id=model_id,
                    data_set_id=data_set_id,
                    name=name,
                    description=description or UNSET,
                ),
            )
        ).id

        if wait_until_complete:
            wait_for_prediction_to_complete(client, prediction_id)

        return Prediction(prediction_id, connection=connection)

    @property
    def is_in_progress(self) -> bool:
        """Specifies whether the prediction is currently running"""
        return self.info.status == Status.IN_PROGRESS

    @property
    def is_ready(self) -> bool:
        """Specifies whether the prediction is finished and available for use"""
        return self.info.status == Status.SUCCEEDED

    def wait_until_ready(self):
        wait_for_prediction_to_complete(self._client, self.prediction_id)

    @property
    def info(self) -> PredictionResponse:
        """Information about the prediction"""
        return self._cached_prediction_info.get_response()

    def as_df(self) -> pd.DataFrame:
        """
        A data frame containing the prediction input data, the predicted output
        and additional information about the predicted output.
        e.g. class confidences for classification predictions.
        The prediction is stored in the column identified by `info.predicted_column_name`.
        """
        _raise_if_prediction_is_not_finished(self.info.status)
        return pd.read_parquet(_download_as_parquet(self._client, self.prediction_id))

    def prediction(self) -> pd.Series:
        """
        A series of predictions, one for each row in the input data set, in the same order as the data set.
        Equivalent to `prediction.as_df()[prediction.info.predicted_column_name]`
        """
        _raise_if_prediction_is_not_finished(self.info.status)
        return self.as_df()[self.info.prediction_column_name]

    def save_as_dataset(
        self, *, name: Optional[str] = None, description: Optional[str] = None
    ) -> DataSet:
        """Save the prediction data as a data set to allow use in other places in the system"""
        _raise_if_prediction_is_not_finished(self.info.status)
        if not self.is_ready:
            raise Exception("Prediction is not available")
        name = name or _default_saved_dataset_name()
        payload = SavePredictionRequest(name=name, description=description)
        data_set_id = check_exists(
            save_prediction.sync(
                client=self._client, id=self.prediction_id, json_body=payload
            )
        ).id
        return load_data_set_from_server(data_set_id, connection=self._connection)

    @property
    def url(self):
        """Get the url of the prediction page in Analyze"""
        base_url = self._connection.base_url
        project_id = self.info.project_id
        prediction_id = self.prediction_id

        return f"{base_url}/{project_id}/predictions/{prediction_id}"

    # TODO: Get prediction forecast - only needed for multiseries / regular forecast models


def load_prediction_from_server(
    prediction_id: int, *, connection: ConnectionLike
) -> Prediction:
    """Load the prediction with the specified id from the Analyze server"""
    return Prediction(prediction_id, connection=connection)


__all__ = [
    "Prediction",
    "load_prediction_from_server",
    "PredictionResponse",
    "PredictionResponseProblemType",
    "PredictionResponseTaskStatus",
    "Status",
    "UNSET",
]



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.csv_parse_options_file_type import CsvParseOptionsFileType
from ..types import UNSET, Unset

T = TypeVar("T", bound="CsvParseOptions")

@attr.s(auto_attribs=True)
class CsvParseOptions:
    """  """
    file_type: CsvParseOptionsFileType
    charset: str = 'UTF-8'
    column_names_in_header: Union[Unset, bool] = False
    separator: Union[Unset, str] = UNSET
    quote_char: Union[Unset, str] = UNSET
    escape_char: Union[Unset, str] = UNSET
    skip_top: Union[Unset, int] = 0
    skip_bottom: Union[Unset, int] = 0
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        file_type = self.file_type.value

        charset =  self.charset
        column_names_in_header =  self.column_names_in_header
        separator =  self.separator
        quote_char =  self.quote_char
        escape_char =  self.escape_char
        skip_top =  self.skip_top
        skip_bottom =  self.skip_bottom

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "fileType": file_type,
            "charset": charset,
        })
        if column_names_in_header is not UNSET:
            field_dict["columnNamesInHeader"] = column_names_in_header
        if separator is not UNSET:
            field_dict["separator"] = separator
        if quote_char is not UNSET:
            field_dict["quoteChar"] = quote_char
        if escape_char is not UNSET:
            field_dict["escapeChar"] = escape_char
        if skip_top is not UNSET:
            field_dict["skipTop"] = skip_top
        if skip_bottom is not UNSET:
            field_dict["skipBottom"] = skip_bottom

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        file_type = CsvParseOptionsFileType(d.pop("fileType"))




        charset = d.pop("charset")

        column_names_in_header = d.pop("columnNamesInHeader", UNSET)

        separator = d.pop("separator", UNSET)

        quote_char = d.pop("quoteChar", UNSET)

        escape_char = d.pop("escapeChar", UNSET)

        skip_top = d.pop("skipTop", UNSET)

        skip_bottom = d.pop("skipBottom", UNSET)

        csv_parse_options = cls(
            file_type=file_type,
            charset=charset,
            column_names_in_header=column_names_in_header,
            separator=separator,
            quote_char=quote_char,
            escape_char=escape_char,
            skip_top=skip_top,
            skip_bottom=skip_bottom,
        )

        csv_parse_options.additional_properties = d
        return csv_parse_options

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .apply_data_prep_steps_request import ApplyDataPrepStepsRequest
from .apply_formula_step import ApplyFormulaStep
from .apply_formula_step_step_type import ApplyFormulaStepStepType
from .cadence import Cadence
from .classification_scorers import ClassificationScorers
from .combine_step import CombineStep
from .combine_step_step_type import CombineStepStepType
from .create_auto_clustering_request import CreateAutoClusteringRequest
from .create_classification_request import CreateClassificationRequest
from .create_cloud_data_set_request import CreateCloudDataSetRequest
from .create_cloud_data_set_request_cloud_type import CreateCloudDataSetRequestCloudType
from .create_cloud_data_set_request_data_origin import (
    CreateCloudDataSetRequestDataOrigin,
)
from .create_clustering_request import CreateClusteringRequest
from .create_data_set_request import CreateDataSetRequest
from .create_db_data_set_request import CreateDbDataSetRequest
from .create_db_data_set_request_data_origin import CreateDbDataSetRequestDataOrigin
from .create_db_data_set_request_db_type import CreateDbDataSetRequestDbType
from .create_deployed_api_client_request import CreateDeployedApiClientRequest
from .create_deployed_model_api_request import CreateDeployedModelApiRequest
from .create_deployed_pipeline_api_request import CreateDeployedPipelineApiRequest
from .create_file_data_set_request import CreateFileDataSetRequest
from .create_forecasting_request import CreateForecastingRequest
from .create_http_data_set_request import CreateHttpDataSetRequest
from .create_http_data_set_request_data_origin import CreateHttpDataSetRequestDataOrigin
from .create_model_response import CreateModelResponse
from .create_multi_forecasting_request import CreateMultiForecastingRequest
from .create_prediction_request import CreatePredictionRequest
from .create_regression_request import CreateRegressionRequest
from .create_test_request import CreateTestRequest
from .csv_parse_options import CsvParseOptions
from .csv_parse_options_file_type import CsvParseOptionsFileType
from .data_partition_method import DataPartitionMethod
from .data_pipeline_create_request import DataPipelineCreateRequest
from .data_pipeline_response import DataPipelineResponse
from .data_pipelines_response import DataPipelinesResponse
from .data_prep_step import DataPrepStep
from .data_prep_step_response import DataPrepStepResponse
from .data_set_response import DataSetResponse
from .data_sets_response import DataSetsResponse
from .data_value import DataValue
from .delete_data_prep_steps_request import DeleteDataPrepStepsRequest
from .deployed_api_client_create_response import DeployedApiClientCreateResponse
from .deployed_api_client_list_response import DeployedApiClientListResponse
from .deployed_api_client_response import DeployedApiClientResponse
from .deployed_model_api_list_response import DeployedModelApiListResponse
from .deployed_model_api_response import DeployedModelApiResponse
from .deployed_pipeline_api_list_response import DeployedPipelineApiListResponse
from .deployed_pipeline_api_response import DeployedPipelineApiResponse
from .discriminators import Discriminators
from .discriminators_column_values_item import DiscriminatorsColumnValuesItem
from .drop_step import DropStep
from .drop_step_step_type import DropStepStepType
from .edit_data_pipeline_request import EditDataPipelineRequest
from .edit_data_set_request import EditDataSetRequest
from .edit_deployed_model_api_request import EditDeployedModelApiRequest
from .edit_deployed_pipeline_api_request import EditDeployedPipelineApiRequest
from .edit_model_request import EditModelRequest
from .edit_prediction_request import EditPredictionRequest
from .edit_test_request import EditTestRequest
from .excel_parse_options import ExcelParseOptions
from .excel_parse_options_file_type import ExcelParseOptionsFileType
from .fill_missing_value_step import FillMissingValueStep
from .fill_missing_value_step_step_type import FillMissingValueStepStepType
from .filter_rows_rule import FilterRowsRule
from .filter_rows_rule_rule import FilterRowsRuleRule
from .filter_rows_step import FilterRowsStep
from .filter_rows_step_step_type import FilterRowsStepStepType
from .future import Future
from .future_outputofyourrequest import FutureOutputofyourrequest
from .group_by_aggregation import GroupByAggregation
from .group_by_aggregation_rule import GroupByAggregationRule
from .group_by_step import GroupByStep
from .group_by_step_step_type import GroupByStepStepType
from .id_response import IdResponse
from .json_parse_options import JsonParseOptions
from .json_parse_options_file_type import JsonParseOptionsFileType
from .merge_columns_step import MergeColumnsStep
from .merge_columns_step_step_type import MergeColumnsStepStepType
from .model_health import ModelHealth
from .model_influence import ModelInfluence
from .model_list_response import ModelListResponse
from .model_response import ModelResponse
from .model_score import ModelScore
from .model_status import ModelStatus
from .model_validation_method import ModelValidationMethod
from .nlp_language import NlpLanguage
from .parse_options import ParseOptions
from .prediction_forecast_response import PredictionForecastResponse
from .prediction_forecast_response_model_forecasts_item import (
    PredictionForecastResponseModelForecastsItem,
)
from .prediction_list_response import PredictionListResponse
from .prediction_response import PredictionResponse
from .prediction_response_problem_type import PredictionResponseProblemType
from .prediction_response_task_status import PredictionResponseTaskStatus
from .problem_type import ProblemType
from .regression_scorers import RegressionScorers
from .rename_step import RenameStep
from .rename_step_step_type import RenameStepStepType
from .replace_step import ReplaceStep
from .replace_step_step_type import ReplaceStepStepType
from .save_prediction_request import SavePredictionRequest
from .save_test_request import SaveTestRequest
from .score_type import ScoreType
from .sentiment_analysis_step import SentimentAnalysisStep
from .sentiment_analysis_step_step_type import SentimentAnalysisStepStepType
from .set_column_level_step import SetColumnLevelStep
from .set_column_level_step_step_type import SetColumnLevelStepStepType
from .set_column_type_step import SetColumnTypeStep
from .set_column_type_step_step_type import SetColumnTypeStepStepType
from .set_time_index_step import SetTimeIndexStep
from .set_time_index_step_step_type import SetTimeIndexStepStepType
from .simple_forecast import SimpleForecast
from .simple_forecast_cadence import SimpleForecastCadence
from .split_column_step import SplitColumnStep
from .split_column_step_step_type import SplitColumnStepStepType
from .status import Status
from .test_list_response import TestListResponse
from .test_response import TestResponse
from .test_response_problem_type import TestResponseProblemType
from .test_response_task_status import TestResponseTaskStatus
from .transform_text_step import TransformTextStep
from .transform_text_step_if_no_match import TransformTextStepIfNoMatch
from .transform_text_step_replace_existing_column import (
    TransformTextStepReplaceExistingColumn,
)
from .transform_text_step_step_type import TransformTextStepStepType



from datetime import datetime
from io import BytesIO
from typing import Optional, Union

import pandas as pd

from ..swagger import Client
from ..swagger.api.tests import create_test, get_test_result_parquet_data, save_test
from ..swagger.models import (
    CreateTestRequest,
    SaveTestRequest,
    ScoreType,
    Status,
    TestResponse,
    TestResponseProblemType,
    TestResponseTaskStatus,
)
from ..swagger.types import UNSET, Unset
from ..utils import (
    CachedTestResponse,
    check_exists,
    check_not_unset,
    wait_for_test_to_complete,
)
from .connection import ConnectionLike, client_from_connectionlike, from_connectionlike
from .data_set import DataSet, load_data_set_from_server


def _default_test_name() -> str:
    return f"Test from Python Client {datetime.utcnow().strftime('%Y/%m/%d %H:%M:%S')}"


def _default_saved_dataset_name() -> str:
    return f"Saved Test from Python Client {datetime.utcnow().strftime('%Y/%m/%d %H:%M:%S')}"


def _download_as_parquet(
    client: Client,
    test_id: int,
) -> BytesIO:
    parquet_bytes = get_test_result_parquet_data.sync_detailed(
        client=client, id=test_id
    ).content
    return BytesIO(parquet_bytes)


def _raise_if_test_is_not_finished(status: Union[Unset, Status]):
    if isinstance(status, Unset):
        raise Exception("Status could not be obtained")

    if status == Status.SUCCEEDED:
        return

    if status == Status.IN_PROGRESS:
        raise Exception("The test is still running")
    if status == Status.FAILED:
        raise Exception("The test has failed")
    raise Exception("The test has not succeeded")


class Test:
    def __init__(self, test_id: int, *, connection: ConnectionLike):
        self.test_id = test_id
        self._connection = from_connectionlike(connection)
        self._client = client_from_connectionlike(connection)
        self._cached_test_info = CachedTestResponse(self._client, self.test_id)

    @classmethod
    def create_test(
        cls,
        model_id: int,
        data_set_id: int,
        *,
        connection: ConnectionLike,
        name: Optional[str] = None,
        description: Optional[str] = None,
        wait_until_complete: bool = True,
    ) -> "Test":
        """
        Create a new test for the specified Analyze model id and data set id.
        Usually you should use the `model.test(data_set)` method rather than this method.
        """
        client = client_from_connectionlike(connection)
        test_id = check_exists(
            create_test.sync(
                client=client,
                json_body=CreateTestRequest(
                    model_id=model_id,
                    data_set_id=data_set_id,
                    name=name or _default_test_name(),
                    description=description or UNSET,
                ),
            )
        ).id

        if wait_until_complete:
            wait_for_test_to_complete(client, test_id)

        return Test(test_id, connection=connection)

    @property
    def is_in_progress(self) -> bool:
        """Specifies whether the test is currently running"""
        return self.info.status == Status.IN_PROGRESS

    @property
    def is_ready(self) -> bool:
        """Specifies whether the test is finished and available for use"""
        return self.info.status == Status.SUCCEEDED

    def wait_until_ready(self):
        wait_for_test_to_complete(self._client, self.test_id)

    @property
    def info(self) -> TestResponse:
        """Information about the test"""
        return self._cached_test_info.get_response()

    def as_df(self) -> pd.DataFrame:
        """
        A data frame containing the test input data, the predicted output and additional information about the predicted output.
        e.g. class confidences for classification predictions.
        The prediction is stored in the column identified by `info.predicted_column_name`.
        """
        _raise_if_test_is_not_finished(self.info.status)
        return pd.read_parquet(_download_as_parquet(self._client, self.test_id))

    def prediction(self) -> pd.Series:
        """
        A series of predictions, one for each row in the input data set, in the same order as the data set.
        Equivalent to `test.as_df()[test.info.predicted_column_name]`
        """
        _raise_if_test_is_not_finished(self.info.status)
        return self.as_df()[self.info.predicted_column_name]

    @property
    def score(self) -> float:
        _raise_if_test_is_not_finished(self.info.status)
        return check_exists(check_not_unset(self.info.score)).value

    def save_as_dataset(
        self, *, name: Optional[str] = None, description: Optional[str] = None
    ) -> DataSet:
        """Save the test data as a data set to allow use in other places in the system"""
        _raise_if_test_is_not_finished(self.info.status)
        if not self.is_ready:
            raise Exception("Test is not available")
        name = name or _default_saved_dataset_name()
        payload = SaveTestRequest(name=name, description=description)
        data_set_id = check_exists(
            save_test.sync(client=self._client, id=self.test_id, json_body=payload)
        ).id
        return load_data_set_from_server(data_set_id, connection=self._connection)

    @property
    def url(self):
        """Get the url of the test page in Analyze"""
        base_url = self._connection.base_url
        project_id = self.info.project_id
        test_id = self.test_id

        return f"{base_url}/{project_id}/tests/{test_id}"

    # TODO: Get test forecast - only needed for multiseries / regular forecast models


def load_test_from_server(test_id: int, *, connection: ConnectionLike) -> Test:
    """Load the test with the specified id from the Analyze server"""
    return Test(test_id, connection=connection)


__all__ = [
    "Test",
    "load_test_from_server",
    "TestResponse",
    "TestResponseProblemType",
    "TestResponseTaskStatus",
    "ScoreType",
]



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.data_prep_step import DataPrepStep
from ..types import UNSET, Unset

T = TypeVar("T", bound="ApplyDataPrepStepsRequest")

@attr.s(auto_attribs=True)
class ApplyDataPrepStepsRequest:
    """  """
    saved_steps_id: Union[Unset, float] = UNSET
    steps: Union[Unset, List[DataPrepStep]] = UNSET
    insertion_index: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        saved_steps_id =  self.saved_steps_id
        steps: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.steps, Unset):
            steps = []
            for steps_item_data in self.steps:
                steps_item = steps_item_data.to_dict()

                steps.append(steps_item)




        insertion_index =  self.insertion_index

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if saved_steps_id is not UNSET:
            field_dict["savedStepsId"] = saved_steps_id
        if steps is not UNSET:
            field_dict["steps"] = steps
        if insertion_index is not UNSET:
            field_dict["insertionIndex"] = insertion_index

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        saved_steps_id = d.pop("savedStepsId", UNSET)

        steps = []
        _steps = d.pop("steps", UNSET)
        for steps_item_data in (_steps or []):
            steps_item = DataPrepStep.from_dict(steps_item_data)



            steps.append(steps_item)


        insertion_index = d.pop("insertionIndex", UNSET)

        apply_data_prep_steps_request = cls(
            saved_steps_id=saved_steps_id,
            steps=steps,
            insertion_index=insertion_index,
        )

        apply_data_prep_steps_request.additional_properties = d
        return apply_data_prep_steps_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ModelStatus(str, Enum):
    CONFIGURING = "CONFIGURING"
    BUILDING = "BUILDING"
    COMPLETE = "COMPLETE"
    FAILED = "FAILED"

    def __str__(self) -> str:
        return str(self.value)


from typing import Any, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="DeployedPipelineApiResponse")

@attr.s(auto_attribs=True)
class DeployedPipelineApiResponse:
    """  """
    id: int
    project_id: int
    name: str
    api_endpoint: str
    documentation_endpoint: str
    description: Union[Unset, Optional[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        id =  self.id
        project_id =  self.project_id
        name =  self.name
        api_endpoint =  self.api_endpoint
        documentation_endpoint =  self.documentation_endpoint
        description =  self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "projectId": project_id,
            "name": name,
            "apiEndpoint": api_endpoint,
            "documentationEndpoint": documentation_endpoint,
        })
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        project_id = d.pop("projectId")

        name = d.pop("name")

        api_endpoint = d.pop("apiEndpoint")

        documentation_endpoint = d.pop("documentationEndpoint")

        description = d.pop("description", UNSET)

        deployed_pipeline_api_response = cls(
            id=id,
            project_id=project_id,
            name=name,
            api_endpoint=api_endpoint,
            documentation_endpoint=documentation_endpoint,
            description=description,
        )

        deployed_pipeline_api_response.additional_properties = d
        return deployed_pipeline_api_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, cast

import attr

from ..models.fill_missing_value_step_step_type import FillMissingValueStepStepType

T = TypeVar("T", bound="FillMissingValueStep")

@attr.s(auto_attribs=True)
class FillMissingValueStep:
    """  """
    columns: List[str]
    default_value: str
    step_type: FillMissingValueStepStepType
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        columns = self.columns




        default_value =  self.default_value
        step_type = self.step_type.value


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "columns": columns,
            "defaultValue": default_value,
            "stepType": step_type,
        })

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        columns = cast(List[str], d.pop("columns"))


        default_value = d.pop("defaultValue")

        step_type = FillMissingValueStepStepType(d.pop("stepType"))




        fill_missing_value_step = cls(
            columns=columns,
            default_value=default_value,
            step_type=step_type,
        )

        fill_missing_value_step.additional_properties = d
        return fill_missing_value_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SimpleForecastCadence(str, Enum):
    HOURLY = "HOURLY"
    DAILY = "DAILY"
    WEEKLY = "WEEKLY"
    MONTHLY = "MONTHLY"
    YEARLY = "YEARLY"

    def __str__(self) -> str:
        return str(self.value)


import io
import json
from typing import Optional, TypeVar, Union

import pandas as pd

from mindfoundry.client.analyze.swagger.types import File, Response, Unset

T = TypeVar("T")


def check_response(response: Response[Optional[T]]) -> T:
    """
    Checks that the response succeeded and has a parseable response body
    :param response: The response to check
    :return: The parsed response body
    """
    if response.status_code >= 200 and response.status_code < 300:
        return check_exists(response.parsed)
    raise Exception(response.content)


def check_exists(response_object: Optional[T]) -> T:
    """
    Checks that the response is not null, and raises an exception if it is
    :param response_object: The object to check
    :return: The non-null value of the object
    """
    if response_object is None:
        raise Exception("No parsed response received")
    return response_object


def check_not_unset(response_object: Union[Unset, T]) -> T:
    """
    Checks that the response is not UNSET, and raises an exception if it is
    :param response_object: The object to check
    :return: The value of the object
    """
    if isinstance(response_object, Unset):
        raise Exception("Expected value was unset")
    return response_object


def get_data_file(data) -> File:
    """
    Convert a range of data inputs into a suitable File object for passing to the API.
    :param data: The data to send - Can be a text or bytes stream, a pandas data frame, or any object to be serialized as json
    """
    if isinstance(data, (io.TextIOBase, io.BytesIO)):
        return File(file_name="input", payload=data)  # type:ignore

    if isinstance(data, pd.DataFrame):
        filename = "./_mf_temp.csv"
        data.to_csv(open(filename, "w"), index=False)
    else:
        filename = "./_mf_temp.json"
        json.dump(data, open(filename, "w"))
    return File(file_name="input", payload=open(filename, "rb"))



import abc
from datetime import datetime
from typing import List, Optional

import pandas as pd

from ....swagger.models import ModelResponse
from ....utils.typing import PathLike
from ...data_set import DataLike
from ...prediction import Prediction
from ...test import Test
from .base import BaseModel


def _default_prediction_name() -> str:
    return f"Prediction from Python Client {datetime.utcnow().strftime('%Y/%m/%d %H:%M:%S')}"


class AnalyzeModel(abc.ABC):
    """Implements the shared methods that call into any internal model regardless of type"""

    def __init__(self, *, _internal_model: BaseModel):
        """Internal constructor - DO NOT USE"""
        self._internal_model = _internal_model

    ########################################################################
    # Methods used or extended by child models to return the correct types #
    ########################################################################

    def _fit(
        self,
        data: DataLike,
        target: str,
        excluded_columns: Optional[List[str]] = None,
        *,
        wait_until_complete: bool = True,
        # Regression specific fit parameters
        order_by: Optional[str] = None,
        partition_by: Optional[str] = None,
        no_mixing: Optional[List[str]] = None,
    ) -> "AnalyzeModel":
        """Internal method - DO NOT USE"""
        # Fit must explicitly override the model with the new fitted model
        self._internal_model = self._internal_model.fit(
            data,
            target,
            excluded_columns,
            order_by=order_by,
            partition_by=partition_by,
            no_mixing=no_mixing,
        )
        if wait_until_complete:
            self._internal_model.wait_until_fitted()
        return self

    ########################################################
    # Generic methods passed through to the internal model #
    ########################################################

    @property
    def model_id(self) -> int:
        """The ID of the model in Analyze"""
        return self._internal_model.model_id

    @property
    def is_fitting(self) -> bool:
        """Specifies whether the model is currently fitting"""
        return self._internal_model.is_fitting()

    @property
    def is_fitted(self) -> bool:
        """Specifies whether the model fitting has succeeded"""
        return self._internal_model.is_fitted()

    @property
    def has_failed_fitting(self) -> bool:
        """Specifies whether the model fitting has failed"""
        return self._internal_model.has_failed_fitting()

    def wait_until_fitted(self):
        """
        Sleep until the model has finished fitting
        """
        self._internal_model.wait_until_fitted()

    def save(self, path: PathLike) -> None:
        """
        Save the model to the specified path
        :param path: The path to which to save the file
        """
        self._internal_model.save(path)

    @property
    def info(self) -> ModelResponse:
        """Get information about the model"""
        return self._internal_model.model_info()

    @property
    def url(self) -> str:
        """Get the url of the model page in Analyze"""
        return self._internal_model.url()

    def predict(self, data) -> pd.Series:
        """
        Use the model to perform a prediction on the supplied data
        and return just the prediction
        :param data: The data to predict
        :return: A series containing the prediction for each row in the data, in the same order as the input data
        """
        return self.predict_object(data).prediction()

    def predict_object(
        self,
        data: DataLike,
        *,
        name: Optional[str] = None,
        description: Optional[str] = None,
        wait_until_complete: bool = True,
    ) -> Prediction:
        """
        Use the model to perform a prediction on the supplied data
        :param data: The data to use to perform the prediction
        :param name: The name to use for the prediction in Analyze
        :param description: The description to use for the prediction in Analyze
        :param wait_until_complete: If True, this method blocks until the model has been fitted and can be used for a prediction.
        :return: An object representing the prediction
        """
        return self._internal_model.predict(
            data,
            name=name or _default_prediction_name(),
            description=description,
            wait_until_complete=wait_until_complete,
        )

    def test(
        self,
        data: DataLike,
        *,
        name: Optional[str] = None,
        description: Optional[str] = None,
        wait_until_complete: bool = True,
    ) -> Test:
        """
        Use the model to perform a test on the supplied data
        :param data: The data to use to perform the test. This must include a column with the Target
        :param name: The name to use for the test in Analyze
        :param description: The description to use for the test in Analyze
        :param wait_until_complete: If True, this method blocks until the model has been fitted and can be used for a test.
        :return: An object representing the test
        """
        return self._internal_model.test(
            data,
            name=name,
            description=description,
            wait_until_complete=wait_until_complete,
        )



import datetime
from typing import Any, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.create_auto_clustering_request import CreateAutoClusteringRequest
from ..models.create_classification_request import CreateClassificationRequest
from ..models.create_clustering_request import CreateClusteringRequest
from ..models.create_forecasting_request import CreateForecastingRequest
from ..models.create_multi_forecasting_request import CreateMultiForecastingRequest
from ..models.create_regression_request import CreateRegressionRequest
from ..models.model_health import ModelHealth
from ..models.model_influence import ModelInfluence
from ..models.model_score import ModelScore
from ..models.model_status import ModelStatus
from ..models.problem_type import ProblemType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ModelResponse")

@attr.s(auto_attribs=True)
class ModelResponse:
    """  """
    id: int
    project_id: int
    name: str
    created_at: datetime.datetime
    created_by: str
    status: ModelStatus
    problem_type: ProblemType
    progress_percent: Optional[float]
    description: Union[Unset, Optional[str]] = UNSET
    failure_details: Union[Unset, Optional[str]] = UNSET
    model_type: Union[Unset, str] = UNSET
    health: Union[Unset, ModelHealth] = UNSET
    score: Union[Optional[ModelScore], Unset] = UNSET
    influences: Union[Unset, List[ModelInfluence]] = UNSET
    create_config: Union[Unset, CreateAutoClusteringRequest, CreateClassificationRequest, CreateClusteringRequest, CreateForecastingRequest, CreateMultiForecastingRequest, CreateRegressionRequest] = UNSET
    create_endpoint: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        id =  self.id
        project_id =  self.project_id
        name =  self.name
        created_at = self.created_at.isoformat()

        created_by =  self.created_by
        status = self.status.value

        problem_type = self.problem_type.value

        description =  self.description
        progress_percent =  self.progress_percent
        failure_details =  self.failure_details
        model_type =  self.model_type
        health: Union[Unset, ModelHealth] = UNSET
        if not isinstance(self.health, Unset):
            health = self.health

        score: Union[None, Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.score, Unset):
            score = self.score.to_dict() if self.score else None

        influences: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.influences, Unset):
            influences = []
            for influences_item_data in self.influences:
                influences_item = influences_item_data.to_dict()

                influences.append(influences_item)




        create_config: Union[Unset, CreateAutoClusteringRequest, CreateClassificationRequest, CreateClusteringRequest, CreateForecastingRequest, CreateMultiForecastingRequest, CreateRegressionRequest]
        if isinstance(self.create_config, Unset):
            create_config = UNSET
        elif isinstance(self.create_config, CreateAutoClusteringRequest):
            create_config = UNSET
            if not isinstance(self.create_config, Unset):
                create_config = self.create_config.to_dict()

        elif isinstance(self.create_config, CreateClassificationRequest):
            create_config = UNSET
            if not isinstance(self.create_config, Unset):
                create_config = self.create_config.to_dict()

        elif isinstance(self.create_config, CreateClusteringRequest):
            create_config = UNSET
            if not isinstance(self.create_config, Unset):
                create_config = self.create_config.to_dict()

        elif isinstance(self.create_config, CreateForecastingRequest):
            create_config = UNSET
            if not isinstance(self.create_config, Unset):
                create_config = self.create_config.to_dict()

        elif isinstance(self.create_config, CreateMultiForecastingRequest):
            create_config = UNSET
            if not isinstance(self.create_config, Unset):
                create_config = self.create_config.to_dict()

        else:
            create_config = UNSET
            if not isinstance(self.create_config, Unset):
                create_config = self.create_config.to_dict()



        create_endpoint =  self.create_endpoint

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "projectId": project_id,
            "name": name,
            "createdAt": created_at,
            "createdBy": created_by,
            "status": status,
            "problemType": problem_type,
            "progressPercent": progress_percent,
        })
        if description is not UNSET:
            field_dict["description"] = description
        if failure_details is not UNSET:
            field_dict["failureDetails"] = failure_details
        if model_type is not UNSET:
            field_dict["modelType"] = model_type
        if health is not UNSET:
            field_dict["health"] = health
        if score is not UNSET:
            field_dict["score"] = score
        if influences is not UNSET:
            field_dict["influences"] = influences
        if create_config is not UNSET:
            field_dict["createConfig"] = create_config
        if create_endpoint is not UNSET:
            field_dict["createEndpoint"] = create_endpoint

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        project_id = d.pop("projectId")

        name = d.pop("name")

        created_at = isoparse(d.pop("createdAt"))




        created_by = d.pop("createdBy")

        status = ModelStatus(d.pop("status"))




        problem_type = ProblemType(d.pop("problemType"))




        description = d.pop("description", UNSET)

        progress_percent = d.pop("progressPercent")

        failure_details = d.pop("failureDetails", UNSET)

        model_type = d.pop("modelType", UNSET)

        health: Union[Unset, ModelHealth] = UNSET
        _health = d.pop("health", UNSET)
        if not isinstance(_health,  Unset):
            health = ModelHealth(_health)




        score = None
        _score = d.pop("score", UNSET)
        if _score is not None and not isinstance(_score,  Unset):
            score = ModelScore.from_dict(_score)




        influences = []
        _influences = d.pop("influences", UNSET)
        for influences_item_data in (_influences or []):
            influences_item = ModelInfluence.from_dict(influences_item_data)



            influences.append(influences_item)


        def _parse_create_config(data: Any) -> Union[Unset, CreateAutoClusteringRequest, CreateClassificationRequest, CreateClusteringRequest, CreateForecastingRequest, CreateMultiForecastingRequest, CreateRegressionRequest]:
            data = None if isinstance(data, Unset) else data
            create_config: Union[Unset, CreateAutoClusteringRequest, CreateClassificationRequest, CreateClusteringRequest, CreateForecastingRequest, CreateMultiForecastingRequest, CreateRegressionRequest]
            try:
                create_config = UNSET
                _create_config = data
                if not isinstance(_create_config,  Unset):
                    create_config = CreateAutoClusteringRequest.from_dict(_create_config)



                return create_config
            except: # noqa: E722
                pass
            try:
                create_config = UNSET
                _create_config = data
                if not isinstance(_create_config,  Unset):
                    create_config = CreateClassificationRequest.from_dict(_create_config)



                return create_config
            except: # noqa: E722
                pass
            try:
                create_config = UNSET
                _create_config = data
                if not isinstance(_create_config,  Unset):
                    create_config = CreateClusteringRequest.from_dict(_create_config)



                return create_config
            except: # noqa: E722
                pass
            try:
                create_config = UNSET
                _create_config = data
                if not isinstance(_create_config,  Unset):
                    create_config = CreateForecastingRequest.from_dict(_create_config)



                return create_config
            except: # noqa: E722
                pass
            try:
                create_config = UNSET
                _create_config = data
                if not isinstance(_create_config,  Unset):
                    create_config = CreateMultiForecastingRequest.from_dict(_create_config)



                return create_config
            except: # noqa: E722
                pass
            create_config = UNSET
            _create_config = data
            if not isinstance(_create_config,  Unset):
                create_config = CreateRegressionRequest.from_dict(_create_config)



            return create_config

        create_config = _parse_create_config(d.pop("createConfig", UNSET))


        create_endpoint = d.pop("createEndpoint", UNSET)

        model_response = cls(
            id=id,
            project_id=project_id,
            name=name,
            created_at=created_at,
            created_by=created_by,
            status=status,
            problem_type=problem_type,
            description=description,
            progress_percent=progress_percent,
            failure_details=failure_details,
            model_type=model_type,
            health=health,
            score=score,
            influences=influences,
            create_config=create_config,
            create_endpoint=create_endpoint,
        )

        model_response.additional_properties = d
        return model_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class Cadence(str, Enum):
    DAILY = "DAILY"
    HOURLY = "HOURLY"
    WEEKLY = "WEEKLY"
    MONTHLY = "MONTHLY"
    YEARLY = "YEARLY"
    BUSINESS_DAILY = "BUSINESS_DAILY"

    def __str__(self) -> str:
        return str(self.value)


import abc
from typing import List, Optional

from ....swagger.models import ModelResponse
from ....utils.typing import PathLike
from ...data_set import DataLike
from ...prediction import Prediction
from ...test import Test


class BaseModel(abc.ABC):
    @abc.abstractmethod
    def fit(
        self,
        data: DataLike,
        target: str,
        excluded_columns: Optional[List[str]] = None,
        *,
        # Classification and Regression specific fit parameters
        order_by: Optional[str] = None,
        partition_by: Optional[str] = None,
        no_mixing: Optional[List[str]] = None,
    ) -> "BaseModel":
        """Fit the model and return a fitted version"""

    @property
    @abc.abstractmethod
    def model_id(self) -> int:
        """The ID of the model in Analyze"""

    @abc.abstractmethod
    def is_fitting(self) -> bool:
        """Specifies whether the model is currently fitting"""

    @abc.abstractmethod
    def is_fitted(self) -> bool:
        """Specifies whether the model fitting has succeeded"""

    @abc.abstractmethod
    def has_failed_fitting(self) -> bool:
        """Specifies whether the model fitting has failed"""

    @abc.abstractmethod
    def save(self, path: PathLike) -> None:
        """Save the model to the specified path"""

    @abc.abstractmethod
    def model_info(self) -> ModelResponse:
        """Get information about the model"""

    @abc.abstractmethod
    def url(self) -> str:
        """Get the URL to the model page on Analyze"""

    @abc.abstractmethod
    def predict(
        self,
        data: DataLike,
        *,
        name: str,
        description: Optional[str],
        wait_until_complete: bool,
    ) -> Prediction:
        """Use the model to perform a prediction on the supplied data"""

    @abc.abstractmethod
    def test(
        self,
        data: DataLike,
        *,
        name: Optional[str] = None,
        description: Optional[str] = None,
        wait_until_complete: bool = True,
    ) -> Test:
        """Use the model to perform a test on the supplied data"""

    @abc.abstractmethod
    def wait_until_fitted(self):
        """Wait for the model to finish and return whether the fitting was successful"""



from typing import Any, Dict, List, Type, TypeVar, cast

import attr

from ..models.merge_columns_step_step_type import MergeColumnsStepStepType

T = TypeVar("T", bound="MergeColumnsStep")

@attr.s(auto_attribs=True)
class MergeColumnsStep:
    """  """
    columns: List[str]
    merged_column_name: str
    step_type: MergeColumnsStepStepType
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        columns = self.columns




        merged_column_name =  self.merged_column_name
        step_type = self.step_type.value


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "columns": columns,
            "mergedColumnName": merged_column_name,
            "stepType": step_type,
        })

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        columns = cast(List[str], d.pop("columns"))


        merged_column_name = d.pop("mergedColumnName")

        step_type = MergeColumnsStepStepType(d.pop("stepType"))




        merge_columns_step = cls(
            columns=columns,
            merged_column_name=merged_column_name,
            step_type=step_type,
        )

        merge_columns_step.additional_properties = d
        return merge_columns_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.group_by_aggregation_rule import GroupByAggregationRule

T = TypeVar("T", bound="GroupByAggregation")

@attr.s(auto_attribs=True)
class GroupByAggregation:
    """  """
    target_column: str
    new_column_name: str
    rule: GroupByAggregationRule
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        target_column =  self.target_column
        new_column_name =  self.new_column_name
        rule = self.rule.value


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "targetColumn": target_column,
            "newColumnName": new_column_name,
            "rule": rule,
        })

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        target_column = d.pop("targetColumn")

        new_column_name = d.pop("newColumnName")

        rule = GroupByAggregationRule(d.pop("rule"))




        group_by_aggregation = cls(
            target_column=target_column,
            new_column_name=new_column_name,
            rule=rule,
        )

        group_by_aggregation.additional_properties = d
        return group_by_aggregation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

from ..models.data_prep_step import DataPrepStep

T = TypeVar("T", bound="DataPipelineResponse")

@attr.s(auto_attribs=True)
class DataPipelineResponse:
    """  """
    id: int
    project_id: int
    name: str
    created_at: datetime.datetime
    created_by: str
    steps: List[DataPrepStep]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        id =  self.id
        project_id =  self.project_id
        name =  self.name
        created_at = self.created_at.isoformat()

        created_by =  self.created_by
        steps = []
        for steps_item_data in self.steps:
            steps_item = steps_item_data.to_dict()

            steps.append(steps_item)





        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "projectId": project_id,
            "name": name,
            "createdAt": created_at,
            "createdBy": created_by,
            "steps": steps,
        })

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        project_id = d.pop("projectId")

        name = d.pop("name")

        created_at = isoparse(d.pop("createdAt"))




        created_by = d.pop("createdBy")

        steps = []
        _steps = d.pop("steps")
        for steps_item_data in (_steps):
            steps_item = DataPrepStep.from_dict(steps_item_data)



            steps.append(steps_item)


        data_pipeline_response = cls(
            id=id,
            project_id=project_id,
            name=name,
            created_at=created_at,
            created_by=created_by,
            steps=steps,
        )

        data_pipeline_response.additional_properties = d
        return data_pipeline_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.filter_rows_rule_rule import FilterRowsRuleRule
from ..types import UNSET, Unset

T = TypeVar("T", bound="FilterRowsRule")

@attr.s(auto_attribs=True)
class FilterRowsRule:
    """  """
    rule: FilterRowsRuleRule
    value1: Union[Unset, str] = UNSET
    inclusive: Union[Unset, bool] = UNSET
    inclusive_between: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        rule = self.rule.value

        value1 =  self.value1
        inclusive =  self.inclusive
        inclusive_between =  self.inclusive_between

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "rule": rule,
        })
        if value1 is not UNSET:
            field_dict["value1"] = value1
        if inclusive is not UNSET:
            field_dict["inclusive"] = inclusive
        if inclusive_between is not UNSET:
            field_dict["inclusiveBetween"] = inclusive_between

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        rule = FilterRowsRuleRule(d.pop("rule"))




        value1 = d.pop("value1", UNSET)

        inclusive = d.pop("inclusive", UNSET)

        inclusive_between = d.pop("inclusiveBetween", UNSET)

        filter_rows_rule = cls(
            rule=rule,
            value1=value1,
            inclusive=inclusive,
            inclusive_between=inclusive_between,
        )

        filter_rows_rule.additional_properties = d
        return filter_rows_rule

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class FilterRowsRuleRule(str, Enum):
    LESS_THAN = "less_than"
    GREATER_THAN = "greater_than"
    BETWEEN = "between"
    NOT_BETWEEN = "not_between"
    EQUALS = "equals"
    NOT_EQUALS = "not_equals"

    def __str__(self) -> str:
        return str(self.value)


from enum import Enum


class ModelValidationMethod(str, Enum):
    TEN_FOLD_CROSS = "TEN_FOLD_CROSS"
    FIVE_FOLD_CROSS = "FIVE_FOLD_CROSS"
    TRAIN_TEST = "TRAIN_TEST"

    def __str__(self) -> str:
        return str(self.value)


# Re-export these so they can be accessed from this package
from mindfoundry.client.analyze.swagger.models import (
    ClassificationScorers,
    CreateAutoClusteringRequest,
    CreateClassificationRequest,
    CreateClusteringRequest,
    CreateForecastingRequest,
    CreateMultiForecastingRequest,
    CreateRegressionRequest,
    DataPartitionMethod,
    ModelHealth,
    ModelInfluence,
    ModelResponse,
    ModelScore,
    ModelStatus,
    ModelValidationMethod,
    NlpLanguage,
    ProblemType,
    RegressionScorers,
    ScoreType,
)

from .classifier import AnalyzeClassifier
from .load_model import load_model, load_model_from_server
from .regressor import AnalyzeRegressor



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.rename_step_step_type import RenameStepStepType

T = TypeVar("T", bound="RenameStep")

@attr.s(auto_attribs=True)
class RenameStep:
    """  """
    column: str
    new_column_name: str
    step_type: RenameStepStepType
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        column =  self.column
        new_column_name =  self.new_column_name
        step_type = self.step_type.value


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "column": column,
            "newColumnName": new_column_name,
            "stepType": step_type,
        })

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        column = d.pop("column")

        new_column_name = d.pop("newColumnName")

        step_type = RenameStepStepType(d.pop("stepType"))




        rename_step = cls(
            column=column,
            new_column_name=new_column_name,
            step_type=step_type,
        )

        rename_step.additional_properties = d
        return rename_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
